<?php
/**
 * Created by PhpStorm.
 * User: snth
 * Date: 10.10.18
 * Time: 13:15
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Wp2leads
 * @subpackage Wp2leads/admin
 */
class Wp2leads_Admin_Ajax {

    public function prepare_data_for_klicktipp_for_cron() {
        $map_id = trim($_POST['mapId']);

        if (empty($map_id)) {
            $response = array('success' => 0, 'error' => 1, 'message' => __('No map ID', 'wp2leads'));

            echo json_encode($response);
            wp_die();
        }

        $existed_cron_maps = json_decode(get_option('wp2leads_cron_maps'), true);

        $current_cron_map = !empty($existed_cron_maps['map_' . $map_id]) ? $existed_cron_maps['map_' . $map_id] : false;

        if (!$current_cron_map) {
            $response = array('success' => 0, 'error' => 1, 'message' => __('No cron settings for map with ID ' . $map_id, 'wp2leads'));

            echo json_encode($response);
            wp_die();
        }

        $current_cron_map_status = $current_cron_map['status'];
        $current_cron_map_date_base = $current_cron_map['date_base'];
        $current_cron_map_has_virtual = false;

        $response = array('success' => 1, 'error' => 0, 'message' => __('Cron maps', 'wp2leads'), 'cronMap' => $current_cron_map);

        foreach ($current_cron_map_date_base as $date) {
            $date_array = explode('.', $date);

            if ($date_array[0] === 'v') {
                $current_cron_map_has_virtual = true;
            }
        }

        $response = array('success' => 1, 'error' => 0, 'message' => __('Cron maps', 'wp2leads'), 'cronMap' => $current_cron_map);

        echo json_encode($response);
        wp_die();

    }

    /**
     * Prepare Data for transfer to KT in Background
     */
    public function prepare_data_for_klicktipp_bg() {
        global $wpdb;
        $data_for_transfer = array();
        $map_id = trim($_POST['mapId']);

        if (!empty($map_id)) {
            $map = MapsModel::get( $map_id );

            $sendApiSettings = trim($_POST['map']);

            $map_to_save = array(
                'map_id'    => $map_id,
                'api'       =>  $sendApiSettings
            );

            $map_global_prefix = !empty($_POST['global_tag_prefix']) ? trim($_POST['global_tag_prefix']) : false;

            if (!$map_global_prefix) {
                delete_option('wp2l_klicktipp_tag_prefix');
            } else {
                $updated = update_option( 'wp2l_klicktipp_tag_prefix', $map_global_prefix );
            }

            $hasSucceeded = MapsModel::update($map_to_save);

            $decodedSendApiSettings = json_decode(stripslashes($sendApiSettings), true);

            $limit = !empty($_POST['limit']) ? $_POST['limit'] : 100000;
            $offset = !empty($_POST['offset']) ? $_POST['offset'] : 0;

            $start = $offset;
            $end = $offset + $limit - 1;

            $saved_results_transient = $wpdb->get_results( "SELECT option_value FROM $wpdb->options WHERE option_name LIKE '%transient_wp2lead_map_to_api_results__". $map_id . '__' . $start . '__' . $end ."%'", ARRAY_A );

            error_log('Started Preparing Data: ' . $start . ' - ' . $end);

            if (count($saved_results_transient) > 0) {
                $results = unserialize($saved_results_transient[0]['option_value']);

                $tags_prefix = ApiHelper::get_map_tags_prefix($map_id);
                $data_for_transfer = ApiHelper::prepareDataForTransfer($decodedSendApiSettings, $results, $tags_prefix);
                set_transient('wp2lead_klicktipp_data_for_transfer__' . $map_id . '__' . $start . '__' . $end, $data_for_transfer, 6 * HOUR_IN_SECONDS);

                $response = array('success' => 1, 'error' => 0, 'message' => __('Ready for transfer', 'wp2leads'), 'count' => count($data_for_transfer), 'result' => $data_for_transfer);

                echo json_encode($response);
                wp_die();
            }

            $response = array('success' => 0, 'error' => 1, 'message' => __('No data for transfer', 'wp2leads'));

            echo json_encode($response);
            wp_die();
        }

        $response = array('success' => 0, 'error' => 1, 'message' => __('No map ID', 'wp2leads'));

        echo json_encode($response);
        wp_die();
    }

    public function get_transfer_modal_data_info() {
        $map_id = trim($_POST['mapId']);

        if (!empty($map_id)) {
            $totally_transfered = StatisticsManager::getTotallyTransferedData($map_id);
            $last_transfered = '';
            $last_transfered_cron = '';
            $crondate_unix = '';

            if ($totally_transfered['time']) {
                $last_transfered = StatisticsManager::convertTimeToLocal($totally_transfered['time']);
            }

            if ($totally_transfered['crontime']) {
                $last_transfered_cron = StatisticsManager::convertTimeToLocal($totally_transfered['crontime']);

                $crondate = new DateTime($last_transfered_cron);
                $crondate_unix = $crondate->format("U");
            }

            $response = array (
                'error' => 0,
                'success' => 1,
                'totally_unique' => $totally_transfered['unique'],
                'totally_all' => $totally_transfered['all'],
                'last_transfered' => $last_transfered,
                'last_transfered_cron' => $last_transfered_cron,
                'last_transfered_cron_unix' => $crondate_unix,
            );

            if ( 0 === $response['available_users'] ) {
                $response['notice']  = '<div class="notice notice-warning inline">';
                $response['notice'] .= '<h4>'.__("There is no data to transfer", "wp2leads").'</h4>';
                $response['notice'] .= '</div>';
            }

            echo json_encode($response);

            wp_die();
        }

        $response = array('success' => 0, 'error' => 1, 'message' => __('No map ID', 'wp2leads'));

        echo json_encode($response);
        wp_die();
    }

    public function load_possible_tags_cloud() {
        $map_id = trim($_POST['mapId']);
        $map = $_POST['map'];
        $email = $_POST['email'];
        $user_data = $_POST['userData'];
        $tags_prefix = trim($_POST['tagsPrefix']);
        $tags_cloud = json_decode(stripslashes($_POST['tagsCloud']), true);

        if ( empty( $_POST['email'] ) ) {
            $response = array(
                'error' => 1,
                'success' => 0,
                'tags' => array(),
                'message' => __('Cannot set tags for current User: <br><strong>Email field below is not filled.</strong>', 'wp2leads'),
            );

            echo json_encode($response);
            wp_die();
        }

        if ($map_id && $map) {

            $kt_tags = array();
            $results = array();
            $results[] = (object) json_decode(stripslashes($user_data), true);

            if ( !empty( $email ) ) {
                $connector = new Wp2leads_KlicktippConnector();
                $logged_in = $connector->login(get_option('wp2l_klicktipp_username'), get_option('wp2l_klicktipp_password'));

                if($logged_in) {
                    $subscriber_id = $connector->subscriber_search($email);

                    if ($subscriber_id) {
                        $subscriber = (array) $connector->subscriber_get($subscriber_id);

                        if (!empty($subscriber['tags'])) {
                            $existed_tags = array();

                            foreach ($subscriber['tags'] as $key => $tag) {
                                $existed_tags[$tag] = !empty($tags_cloud[$tag]) ? $tags_cloud[$tag] : '';

                                if (!$existed_tags[$tag]) {
                                    unset($subscriber['tags'][$key]);
                                }
                            }

                            $kt_tags = array_merge($kt_tags, $subscriber['tags']);
                        }
                    }
                }
            }

            $decoded_map = json_decode(stripslashes($map), true);
            $data_for_transfer = ApiHelper::prepareDataForTransfer($decoded_map, $results, $tags_prefix);
            $tags = $data_for_transfer[$email]['tags'];

            foreach ($tags as $key => $tag) {
                $tag = str_replace('&#38;', '+', $tag);
                $tag = str_replace('&amp;', '+', $tag);
                $tag = str_replace('&', '+', $tag);

                $tags[$key] = $tag;
            }

            $manually_tags = $data_for_transfer[$email]['manually_tags'];
            $detach_tags = array_unique(array_merge($data_for_transfer[$email]['detach_tags'], $decoded_map['detach_tags']['tag_ids']));

            unset($email);
            unset($map);
            unset($map);
            unset($map);

            ob_start();

            include_once 'partials/ajax/possible-tags-cloud.php';
            $possible_tags_cloud = ob_get_clean();

            $response = array(
                'success' => 1,
                'error' => 0,
                'message' => __('Success', 'wp2leads'),
                'kt_tags' => $kt_tags,
                'tags' => $tags,
                'manually_tags' => $manually_tags,
                'detach_tags' => $detach_tags,
                'possible_tags_cloud' => $possible_tags_cloud,
            );

            echo json_encode($response);
            wp_die();
        }

        $response = array('success' => 0, 'error' => 1, 'message' => __('No map ID', 'wp2leads'));

        echo json_encode($response);
        wp_die();
    }

    /**
     * Prepare data for Klick Tip
     */
    public function ajax_prepare_data_for_klicktipp() {
        $data_for_transfer = array();
        $map_id = trim($_POST['map_id']);

        if (!empty($map_id)) {
            $result_limit = MapBuilderManager::get_map_query_results_limit();

            if (!$result_limit) {
                $result_limit = 1000;
            }

            $map = MapsModel::get( $map_id );
            $decodedMap = unserialize($map->mapping);
            $decodedApiSettings = unserialize($map->api);
            $sendApiSettings = trim($_POST['map']);
            $decodedSendApiSettings = json_decode(stripslashes($sendApiSettings), true);

            $prepared_map = $map;
            $prepared_map->api = serialize($decodedSendApiSettings);


            $results = MapsModel::get_map_query_results($decodedMap, $result_limit, 0);
            $tags_prefix = ApiHelper::get_map_tags_prefix($map_id);
            $data_for_transfer = ApiHelper::prepareDataForTransfer($decodedSendApiSettings, $results, $tags_prefix);

            set_transient('klicktipp_data_for_transfer', $data_for_transfer);
        }

        $totally_transfered = StatisticsManager::getTotallyTransferedData($map_id);
        // $available_users = count( $data_for_transfer ) - ApiHelper::getSubscribersCounter($map_id)['unique'];
        $available_users = count( $data_for_transfer );
        $last_transfered = '';
        $last_transfered_cron = '';
        $crondate_unix = '';

        if ($totally_transfered['time']) {
            $last_transfered = StatisticsManager::convertTimeToLocal($totally_transfered['time']);
        }

        if ($totally_transfered['crontime']) {
            $last_transfered_cron = StatisticsManager::convertTimeToLocal($totally_transfered['crontime']);

            $crondate = new DateTime($last_transfered_cron);
            $crondate_unix = $crondate->format("U");
        }

        $response = array (
            'error' => 0,
            'success' => 1,
            'available_users' => $available_users > 0 ? $available_users : 0,
            'totally_unique' => $totally_transfered['unique'],
            'totally_all' => $totally_transfered['all'],
            'last_transfered' => $last_transfered,
            'last_transfered_cron' => $last_transfered_cron,
            'last_transfered_cron_unix' => $crondate_unix,
        );

        if ( 0 === $response['available_users'] ) {
            $response['notice']  = '<div class="notice notice-warning inline">';
            $response['notice'] .= '<h4>'.__("There is no data to transfer", "wp2leads").'</h4>';
            $response['notice'] .= '</div>';
        }

        echo json_encode($response);

        wp_die();
    }

    public function get_map_rows_count() {
        $map_id = trim($_POST['mapId']);

        $map = MapsModel::get($map_id);

        $rows_count = MapsModel::get_map_query_rows_count(unserialize($map->mapping));

        echo json_encode(array('error' => 0, 'success' => 1, 'message' => count($rows_count)));

        wp_die();
    }

    public function transfer_all_data_to_klicktip_bg() {
        global $wpdb;
        $map_id = trim($_POST['mapId']);
        $sendApiSettings = trim($_POST['map']);

        if ( !empty($map_id) && Wp2leads_License::is_map_transfer_allowed($map_id) ) {
            $map_to_save = array(
                'map_id'    => $map_id,
                'api'       =>  $sendApiSettings
            );

            $hasSucceeded = MapsModel::update($map_to_save);
            $limit = !empty($_POST['limit']) ? $_POST['limit'] : 100000;
            $offset = !empty($_POST['offset']) ? $_POST['offset'] : 0;

            $start = $offset;
            $end = $offset + $limit - 1;

            $saved_prepared_data_transient = $wpdb->get_results( "SELECT option_value FROM $wpdb->options WHERE option_name LIKE '%transient_wp2lead_klicktipp_data_for_transfer__". $map_id . '__' . $start . '__' . $end ."%'", ARRAY_A );

            if (count($saved_prepared_data_transient) > 0) {
                $data_for_transfer = unserialize($saved_prepared_data_transient[0]['option_value']);

                $count = Wp2leads_Background_MapToApi::maptoapi_bg($map_id, $data_for_transfer);

                $response = array('success' => 1, 'error' => 0, 'message' => __('Ready for transfer', 'wp2leads'), 'count' => $count, 'result' => $data_for_transfer);

                echo json_encode($response);
                wp_die();
            }

            $response = array('success' => 0, 'error' => 1, 'message' => __('No data to transfer', 'wp2leads'));

            echo json_encode($response);
            wp_die();
        }

        $response = array('success' => 0, 'error' => 1, 'message' => __('No map ID', 'wp2leads'));

        echo json_encode($response);
        wp_die();
    }

    public function transfer_to_klicktipp_bg() {
        $map_id = trim($_POST['map_id']);

        if ( !empty($map_id) && Wp2leads_License::is_map_transfer_allowed($map_id) ) {
            $data_for_transfer = get_transient('klicktipp_data_for_transfer');

            if ( empty( $data_for_transfer ) ) {

                echo json_encode(['error' => 1, 'success' => 0]);
                wp_die();
            }

            $count = Wp2leads_Background_MapToApi::maptoapi_bg($map_id, $data_for_transfer);

            echo json_encode(['error' => 0, 'success' => 1, 'message' => __($count . ' run to transfer in background', 'wp2leads')]);
            wp_die();
        }

        $response = array('success' => 0, 'error' => 1, 'message' => __('No map ID', 'wp2leads'));

        echo json_encode($response);
        wp_die();
    }

    public function terminate_all_map_to_api() {
        $wp2l_map_to_api_in_progress = get_transient( 'wp2leads_maptoapi_bg_in_process' );
        $terminate_errors = array();
        $terminate_success = array();

        if ($wp2l_map_to_api_in_progress) {
            foreach ($wp2l_map_to_api_in_progress as $map_id => $process) {
                $map_object = MapsModel::get($map_id);
                $terminate_result = ApiHelper::terminate_map_to_api($map_id);

                if (!$terminate_result) {
                    $terminate_errors[$map_id] = $map_object->name;
                } else {
                    $terminate_success[$map_id] = $map_object->name;
                }
            }

            $response = array('success' => 1, 'error' => 0, 'message' => __('Map with ID  successfully terminated', 'wp2leads'));

            echo json_encode($response);
            wp_die();
        }

        $response = array('success' => 0, 'error' => 1, 'message' => __('No background transfer currently running.', 'wp2leads'));

        echo json_encode($response);
        wp_die();
    }

    public function refresh_all_map_to_api() {
        $wp2l_map_to_api_in_progress = get_transient( 'wp2leads_maptoapi_bg_in_process' );

        ob_start();

        if ($wp2l_map_to_api_in_progress) {
            include plugin_dir_path(  WP2LEADS_PLUGIN_FILE ) . 'admin/partials/ajax/running-background-processes.php';
        } else {
            ?>
            <p><strong><?php _e('WP2Leads', 'wp2leads') ?></strong>: <?php _e('No background transfer currently running.', 'wp2leads') ?></p>
            <?php
        }

        $html = ob_get_clean();

        $response = array('success' => 1, 'error' => 0, 'html' => $html);

        echo json_encode($response);
        wp_die();
    }

    public function get_map_to_api_statistics() {

        ob_start();

        include plugin_dir_path(  WP2LEADS_PLUGIN_FILE ) . 'admin/partials/wp2leads-admin-maptoapi-bg-in-process-notices.php';

        $html = ob_get_clean();

        $response = array('success' => 1, 'error' => 0, 'html' => $html);

        echo json_encode($response);
        wp_die();
    }

    public function terminate_map_to_api() {
        global $wpdb;

        $map_id = trim($_POST['mapId']);

        if ( empty($map_id) ) {
            $response = array('success' => 0, 'error' => 1, 'message' => __('No map ID', 'wp2leads'));

            echo json_encode($response);
            wp_die();
        }

        $terminate_result = ApiHelper::terminate_map_to_api($map_id);

        if (!$terminate_result) {
            $response = array('success' => 0, 'error' => 1, 'message' => __('Something went wrong', 'wp2leads'));

            echo json_encode($response);
            wp_die();
        }

        $response = array('success' => 1, 'error' => 0, 'message' => __('Map with ID ' . $map_id . ' successfully terminated', 'wp2leads'));

        echo json_encode($response);
        wp_die();
    }

    public function update_existed_tag_fieldset_list() {
        $map_id = trim($_POST['mapId']);
        $sendApiSettings = trim($_POST['map']);
        $decodedSendApiSettings = json_decode(stripslashes($sendApiSettings), true);

        if ( empty($map_id) ) {
            $response = array('success' => 0, 'error' => 1, 'message' => __('No map ID', 'wp2leads'));

            echo json_encode($response);
            wp_die();
        }

        $connector = new Wp2leads_KlicktippConnector();
        $logged_in = $connector->login();
        $connector->get_last_error();

        if ($logged_in) {
            $fields = $connector->field_index();
            $tags = (array) $connector->tag_index();
            asort($tags, SORT_STRING | SORT_FLAG_CASE | SORT_NATURAL);
            $optins = $connector->subscription_process_index();
        }

        ob_start();
        include_once 'partials/ajax/existet-tag-fieldset-list.php';
        $tags_list = ob_get_clean();

        ob_start();
        include_once 'partials/ajax/existet-detach-tag-fieldset-list.php';
        $detach_tags_list = ob_get_clean();

        $response = array(
            'success' => 1,
            'error' => 0,
            'message' => __('Success', 'wp2leads'),
            'tags_list' => $tags_list,
            'detach_tags_list' => $detach_tags_list
        );

        echo json_encode($response);
        wp_die();


    }

    public function transfer_current_to_klicktipp() {
        global $wpdb;
        $map_id = trim($_POST['mapId']);
        $current_email = trim($_POST['email']);
        $current_data = $_POST['mapResult'];
        $sendApiSettings = trim($_POST['map']);

        if ( !Wp2leads_License::is_map_transfer_allowed($map_id) ) {

            $response = array('success' => 0, 'error' => 1, 'message' => __('Transfer are not allowed', 'wp2leads'));

            echo json_encode($response);
            wp_die();
        }

        if ( empty($map_id) ) {
            $response = array('success' => 0, 'error' => 1, 'message' => __('No map ID', 'wp2leads'));

            echo json_encode($response);
            wp_die();
        }

        if ( empty($current_email) ) {
            $response = array('success' => 0, 'error' => 1, 'message' => __('No user email', 'wp2leads'));

            echo json_encode($response);
            wp_die();
        }

        if ( !empty($map_id) && Wp2leads_License::is_map_transfer_allowed($map_id) ) {

            $map_global_prefix = !empty($_POST['global_tag_prefix']) ? trim($_POST['global_tag_prefix']) : false;

            if (!$map_global_prefix) {
                delete_option('wp2l_klicktipp_tag_prefix');
            } else {
                $updated = update_option( 'wp2l_klicktipp_tag_prefix', $map_global_prefix );
            }

            $map_to_save = array(
                'map_id'    => $map_id,
                'api'       =>  $sendApiSettings
            );

            $hasSucceeded = MapsModel::update($map_to_save);

            $results = array();
            $results[] = (object) $current_data;
            $decodedSendApiSettings = json_decode(stripslashes($sendApiSettings), true);
            $tags_prefix = ApiHelper::get_map_tags_prefix($map_id);
            $data_for_transfer = ApiHelper::prepareDataForTransfer($decodedSendApiSettings, $results, $tags_prefix);
            $result = KlickTippManager::transfer_data_to_kt($map_id, $data_for_transfer);

            if ( $result ) {
                $totally_transfered = StatisticsManager::getTotallyTransferedData($map_id);
                $new_subscribers_amount = $result['added_subscribers'];
                $updated_subscribers_amount = $result['existed_subscribers'];
                $total_transferred = $new_subscribers_amount + $updated_subscribers_amount;
                $last_transferred_time = ApiHelper::convertTimeToLocal($result['last_transferred_time']);

                echo json_encode([
                    'error' => 0,
                    'success' => 1,
                    'added_subscribers' => $new_subscribers_amount,
                    'existed_subscribers' => $updated_subscribers_amount,
                    'total_transferred' => $total_transferred,
                    'last_transferred_time' => $last_transferred_time,
                    'totally_unique' => $totally_transfered['unique'],
                    'totally_all' => $totally_transfered['all'],
                ]);

                wp_die();
            } else {
                echo json_encode(array('error' => 1, 'success' => 0, 'message' => __('You can not transfer this user', 'wp2leads')));
                wp_die();
            }

            $response = array('success' => 0, 'error' => 1, 'message' => __('No data to transfer', 'wp2leads'));

            echo json_encode($response);
            wp_die();
        }

        $response = array('success' => 0, 'error' => 1, 'message' => __('No map ID', 'wp2leads'));

        echo json_encode($response);
        wp_die();
    }

    /**
     * Transfer data to Klick Tipp
     */
    public function ajax_transfer_to_klicktipp() {

        $map_id = trim($_POST['map_id']);
        $is_transfer_allowed = Wp2leads_License::is_map_transfer_allowed($map_id);

        if ($is_transfer_allowed && !empty($map_id)) {
            $data_for_transfer = get_transient('klicktipp_data_for_transfer');

            if (!empty($_POST['email'])) {
                $data_for_transfer = ApiHelper::getCurrentUserData($data_for_transfer, $_POST['email']);
            }

            if ( empty( $data_for_transfer ) ) {
                echo json_encode(['error' => 1, 'success' => 0]);
                wp_die();
            }

            $result = KlickTippManager::transfer_data_to_kt($_POST['map_id'], $data_for_transfer);

            if ( $result ) {
                $totally_transfered = StatisticsManager::getTotallyTransferedData($map_id);
                $new_subscribers_amount = $result['added_subscribers'];
                $updated_subscribers_amount = $result['existed_subscribers'];
                $total_transferred = $new_subscribers_amount + $updated_subscribers_amount;
                $last_transferred_time = ApiHelper::convertTimeToLocal($result['last_transferred_time']);

                echo json_encode([
                    'error' => 0,
                    'success' => 1,
                    'added_subscribers' => $new_subscribers_amount,
                    'existed_subscribers' => $updated_subscribers_amount,
                    'total_transferred' => $total_transferred,
                    'last_transferred_time' => $last_transferred_time,
                    'totally_unique' => $totally_transfered['unique'],
                    'totally_all' => $totally_transfered['all'],
                ]);

                wp_die();
            } else {
                echo json_encode(array('error' => 1, 'success' => 0, 'message' => __('You can not transfer this user', 'wp2leads')));
                wp_die();
            }
        } else {
            echo json_encode(array('error' => 1, 'success' => 0, 'message' => __('Transfer is not allowed', 'wp2leads')));
            wp_die();
        }
    }

    public function import_maps() {
        $mapimportids = !empty($_POST['mapimportids']) ? $_POST['mapimportids'] : false;

        if (!$mapimportids || count($mapimportids) === 0) {
            $response = array(
                'error' => 1,
                'success' => 0,
                'message' => __('Please select at least one map to import', 'wp2leads'),
            );

            echo json_encode($response);
            wp_die();
        }

        $mapids = array();

        foreach ($mapimportids as $mapimportid) {
            $mapids[] = $mapimportid['mapId'];
        }

        $result = MapBuilderManager::import_maps_from_server($mapids);

        if (!$result) {
            $response = array(
                'error' => 1,
                'success' => 0,
                'message' => __('Something went wrong, please try later', 'wp2leads'),
            );

            echo json_encode($response);
            wp_die();
        }

        $response = array(
            'error' => 0,
            'success' => 1,
            'message' => $result . __(' Maps successfuly imported from server', 'wp2leads'),
        );

        echo json_encode($response);
        wp_die();
    }

    public function import_pending_maps() {
        $mapimportids = !empty($_POST['mapimportids']) ? $_POST['mapimportids'] : false;

        if (!$mapimportids || count($mapimportids) === 0) {
            $response = array(
                'error' => 1,
                'success' => 0,
                'message' => __('Please select at least one map to import', 'wp2leads'),
            );

            echo json_encode($response);
            wp_die();
        }

        $mapids = array();

        foreach ($mapimportids as $mapimportid) {
            $mapids[] = $mapimportid['mapId'];
        }

        $result = MapBuilderManager::import_pending_maps_from_server($mapids);

        if (!$result) {
            $response = array(
                'error' => 1,
                'success' => 0,
                'message' => __('Something went wrong, please try later', 'wp2leads'),
            );

            echo json_encode($response);
            wp_die();
        }

        $response = array(
            'error' => 0,
            'success' => 1,
            'message' => $result . __(' Maps successfuly imported from server', 'wp2leads'),
        );

        echo json_encode($response);
        wp_die();
    }

    public function export_maps() {
        $mapuploadids = !empty($_POST['mapuploadids']) ? $_POST['mapuploadids'] : false;
        $mapupdateids = !empty($_POST['mapupdateids']) ? $_POST['mapupdateids'] : false;

        if ($mapuploadids) {
            $count = count($mapuploadids);
            $result = MapBuilderManager::export_maps_to_server($mapuploadids);

            $response = array(
                'error' => 0,
                'success' => 1,
                'message' => $count . __(' Maps successfuly uploaded to server', 'wp2leads'),
            );

            echo json_encode($response);
            wp_die();
        }

        if ($mapupdateids) {
            $count = count($mapupdateids);
            $result = MapBuilderManager::update_maps_on_server($mapupdateids);

            $response = array(
                'error' => 0,
                'success' => 1,
                'message' => $count . __(' Maps successfuly updatet on server', 'wp2leads'),
            );

            echo json_encode($response);
            wp_die();
        }

        $response = array(
            'error' => 1,
            'success' => 0,
            'message' => __('Please select at least one map to export', 'wp2leads'),
        );

        echo json_encode($response);
        wp_die();
    }

    public function import_from_remote() {

        $request = wp_remote_get(base64_decode('aHR0cHM6Ly93cDJsZWFkcy1mb3Ita2xpY2stdGlwcC5jb20vc2VydmVyL2RsL3dwMmxlYWRzLW1hcHMuanNvbg=='));

        if( is_wp_error( $request ) ) {
            $response = array(
                'error' => 1,
                'success' => 0,
                'message' => __('Something went wrong', 'wp2leads'),
            );

            echo json_encode($response);
            wp_die();
        }

        $body = wp_remote_retrieve_body( $request );
        $decoded = json_decode( $body );
        $error = false;

        if (json_last_error() === JSON_ERROR_NONE) {
            if(MapBuilderManager::contains_valid_json_maps($decoded)) {
                MapBuilderManager::ingest_uploaded_json_maps($decoded);
            } else {
                $error = true;
            }
        } else {
            $error = true;
        }

        if($error) {
            $response = array(
                'error' => 1,
                'success' => 0,
                'message' => __('Something went wrong', 'wp2leads'),
            );

            echo json_encode($response);
            wp_die();
        }

        $maps_count = count($decoded);

        $response = array(
            'error' => 0,
            'success' => 1,
            'message' => $maps_count . __(' maps imported successfuly', 'wp2leads'),
        );

        echo json_encode($response);
        wp_die();
    }

    public function maps_actions_run() {

        $action = !empty($_POST['run']) ? sanitize_text_field($_POST['run']) : false;
        $maps_ids = !empty($_POST['maps']) ? $_POST['maps'] : false;

        if (!$action) {
            $response = array(
                'error' => 1,
                'success' => 0,
                'message' => __('Please, select an action', 'wp2leads'),
            );

            echo json_encode($response);
            wp_die();
        }

        if (!$maps_ids) {
            $response = array(
                'error' => 1,
                'success' => 0,
                'message' => __('Please, select at least one map', 'wp2leads'),
            );

            echo json_encode($response);
            wp_die();
        }

        $maps_count = count($maps_ids);

        foreach ($maps_ids as $map_id) {
            $result = MapsModel::delete($map_id);
        }

        $response = array(
            'error' => 0,
            'success' => 1,
            'message' => $maps_count . __(' map(s) deleted successfuly', 'wp2leads'),
        );

        echo json_encode($response);
        wp_die();
    }

    public function ajax_get_subscriber_tags_from_klicktipp() {
        $response = array(
            'error' => 1,
            'success' => 0,
            'tags' => array(),
            'message' => __('Cannot get tags for current User: ', 'wp2leads'),
        );

        if ( !empty( $_POST['email'] ) ) {
            $email = trim($_POST['email']);
            $connector = new Wp2leads_KlicktippConnector();
            $logged_in = $connector->login(get_option('wp2l_klicktipp_username'), get_option('wp2l_klicktipp_password'));

            if($logged_in) {
                $subscriber_id = $connector->subscriber_search($email);

                if ($subscriber_id) {
                    $available_tags = $connector->tag_index();
                    $subscriber = (array) $connector->subscriber_get($subscriber_id);

                    if (!empty($subscriber['tags'])) {
                        $existed_tags = array();

                        foreach ($subscriber['tags'] as $key => $tag) {
                            $existed_tags[$tag] = !empty($available_tags[$tag]) ? $available_tags[$tag] : '';

                            if (!$existed_tags[$tag]) {
                                unset($subscriber['tags'][$key]);
                            }
                        }

                        $response['tags'] = array_merge($response['tags'], $subscriber['tags']);

                        $response['error'] = 0;
                        $response['success'] = 1;
                    }
                } else {

                    $response['error'] = 1;
                    $response['success'] = 0;
                    $response['message'] .= __('<br><strong>User is not in Klick-Tipp.</strong>', 'wp2leads');
                }
            }
        } else {
            $response['error'] = 1;
            $response['success'] = 0;
            $response['message'] .= ' ' . __('<br><strong>Email field below is not filled.</strong>', 'wp2leads');
        }

        echo json_encode($response);
        wp_die();
    }

    public function ajax_save_new_map()
    {
        if ($_POST['map_id']) {
            $map_global_prefix = !empty($_POST['global_tag_prefix']) ? trim($_POST['global_tag_prefix']) : false;

            if (!$map_global_prefix) {
                delete_option('wp2l_klicktipp_tag_prefix');
            } else {
                $updated = update_option( 'wp2l_klicktipp_tag_prefix', $map_global_prefix );
            }

            $hasSucceeded = MapsModel::update($_POST);

            global $wpdb;
            $wpdb->query( "DELETE FROM $wpdb->options WHERE option_name LIKE '%wp2lead_map_to_api_results__". $_POST['map_id'] ."%'" );
        } else {
            $hasSucceeded = MapsModel::create($_POST);
        }

        if ($hasSucceeded) {
            echo json_encode(['error' => 0, 'success' => 1, 'map_id' => $hasSucceeded['map_id'], 'mapping' => $hasSucceeded['mapping']]);
        } else {
            echo json_encode(['error' => 1, 'success' => 0, 'map_id' => null]);
        }

        wp_die();
    }

    public function ajax_map_delete()
    {
        if (MapsModel::delete($_POST['map_id'])) {
            echo json_encode(['error' => 0, 'success' => 1]);
        } else {
            echo json_encode(['error' => 1, 'success' => 0]);
        }

        wp_die();
    }

    public function ajax_get_another_page_data() {
        if ('POST' === $_SERVER['REQUEST_METHOD']) {
            if (!isset($_POST['page_number']) || !isset($_POST['direction']) || !isset($_POST['active_mapping'])) {
                wp_die();
            }

            $offset = $_POST['page_number'] - 1;
            $rows_count = intval($_POST['rows_count']);

            if ($offset < 0) {
                $offset = 0;
            }

            $decodedMap = (array) unserialize(get_transient('wp2leads_map_' . $_POST['active_mapping']));

            $all_results = MapsModel::get_map_query_results($decodedMap, 1000, 0);

            $results = array(
                $all_results[$offset]
            );

            $paths = ApiHelper::get_paths($results, $decodedMap);

            ob_start();
            $next_page = $offset + 2;

            if ($next_page > ($rows_count + 1)) {
                $next_page = $rows_count + 1;
            }

            $prev_page = $next_page - 2;

            include_once 'partials/ajax/available_options.php';
            $available_options = ob_get_clean();

            echo $available_options;
            wp_die();
        }
    }

    public function get_map_query_results_limit() {
        $limit = MapBuilderManager::get_map_query_results_limit();
        $result = array('success' => 1, 'limit' => $limit);

        echo json_encode($result);
        die();
    }

    public function get_available_options_data() {

        $map_id = $_POST['mapId'];
        $page_number = $_POST['pageNumber'];
        $map_result = $_POST['mapResult'];
        $rows_count = $_POST['count'];
        $decodedMap = (array) unserialize(get_transient('wp2leads_map_' . $map_id));
        $paths = ApiHelper::get_options_paths($map_result, $decodedMap);

        if (!$page_number) {
            $next_page = 2;
            $prev_page = 0;
        } else {
            $next_page = $page_number + 1;
            $prev_page = $page_number - 1;
        }

        ob_start();

        include_once 'partials/ajax/available_options_page.php';
        $available_options = ob_get_clean();

        echo json_encode(array('error' => 0, 'success' => 1, 'message' => __('Transfer is not allowed', 'wp2leads'), 'pageNumber' => $page_number, 'mapResult' => $map_result, 'availableOptions' => $available_options));
        wp_die();
    }

    public function get_map_query_results_by_map_id_limited() {
        global $wpdb;

        $map_id = !empty($_POST['mapId']) ? $_POST['mapId'] : false;

        if (!$map_id) {
            $response = array('success' => 0, 'error' => 1, 'result' => 'Select map ID');
            echo json_encode($response);

            wp_die();
        }

        $limit = !empty($_POST['limit']) ? $_POST['limit'] : 100000;
        $offset = !empty($_POST['offset']) ? $_POST['offset'] : 0;

        $start = $offset;
        $end = $offset + $limit - 1;

        $map = MapsModel::get($map_id);
        $mapping = unserialize($map->mapping);
        $result = MapsModel::get_map_query_results($mapping, $limit, $offset);
        $count = count($result);

        $response = array('success' => 1, 'error' => 0, 'result' => $result);

        echo json_encode($response);
        wp_die();
    }

    public function get_map_query_results_by_map_id() {
        global $wpdb;

        $map_id = !empty($_POST['mapId']) ? $_POST['mapId'] : false;

        if (!$map_id) {
            $response = array('success' => 0, 'error' => 1, 'result' => 'Select map ID');
            echo json_encode($response);

            wp_die();
        }

        $limit = !empty($_POST['limit']) ? $_POST['limit'] : 100000;
        $offset = !empty($_POST['offset']) ? $_POST['offset'] : 0;

        $start = $offset;
        $end = $offset + $limit - 1;

        $saved_results_transient = $wpdb->get_results( "SELECT option_value FROM $wpdb->options WHERE option_name LIKE '%transient_wp2lead_map_to_api_results__". $map_id . '__' . $start . '__' . $end ."%'", ARRAY_A );

        if (count($saved_results_transient) > 0) {
            $result = unserialize($saved_results_transient[0]['option_value']);
        } else {
            $map = MapsModel::get($map_id);
            $mapping = unserialize($map->mapping);
            $result = MapsModel::get_map_query_results($mapping, $limit, $offset);
            set_transient('wp2lead_map_to_api_results__' . $map_id . '__' . $start . '__' . $end, $result, 3 * HOUR_IN_SECONDS);
        }

        $count = count($result);
        //$end = $offset + $count - 1;

        $memory = (memory_get_usage()/1048576);

        // $saved_results_transients = $wpdb->get_results( "SELECT option_name FROM $wpdb->options WHERE option_name LIKE '%wp2lead_map_to_api_results__". $map_id ."%'" );

        $response = array('success' => 1, 'error' => 0, 'result' => $result);

        echo json_encode($response);
        wp_die();
    }

    public function ajax_fetch_map_query_results()
    {
        $map = !empty($_POST['map']) ? $_POST['map'] : false;

        if (!$map) {
            $map_id = !empty($_POST['map_id']) ? $_POST['map_id'] : false;

            $map_object = MapsModel::get($map_id);
            $map = unserialize($map_object->mapping);
        }

        $results = MapsModel::get_map_query_results($map, $_POST['limit'] ? $_POST['limit'] : 100000, $_POST['offset'] ? $_POST['offset'] : 0);

        echo json_encode($results);
        wp_die();
    }

    public function ajax_get_mapping() {
        $mapping = array();

        if (isset($_POST['map_id']) && $activeMap = MapsModel::get($_POST['map_id'])) {
            $mapping = unserialize($activeMap->mapping);
        }

        foreach ($mapping['selects'] as $key => $select) {
            if (isset($mapping['excludes']) && in_array($select, $mapping['excludes'])) {
                unset($mapping['selects'][$key]);
            }
        }

        $mapping['selects'] = array_values($mapping['selects']);

        echo json_encode($mapping);
        die();
    }

    /**
     * Ajax license activation
     */
    public function ajax_license_activation() {
        $license_email = isset($_POST['licenseEmail']) ? $_POST['licenseEmail'] : '';
        $license_key = isset($_POST['licenseKey']) ? $_POST['licenseKey'] : '';

        $result = Wp2leads_License::activate_license($license_email, $license_key);

        echo json_encode($result);
        die();
    }

    /**
     * Ajax license activation
     */
    public function ajax_modal_license_activation() {
        $license_email = isset($_POST['licenseEmail']) ? $_POST['licenseEmail'] : '';
        $license_key = isset($_POST['licenseKey']) ? $_POST['licenseKey'] : '';

        $result = Wp2leads_License::activate_license($license_email, $license_key);

        if ($result['success'] === 1 && $result['status'] === 200) {
            $complete = Wp2leads_License::complete_license_activation();
        }

        echo json_encode($result);
        die();
    }

    /**
     *  Ajax license deactivation
     */
    public function ajax_license_deactivation() {
        $license_email = isset($_POST['licenseEmail']) ? $_POST['licenseEmail'] : '';
        $license_key = isset($_POST['licenseKey']) ? $_POST['licenseKey'] : '';

        $result = Wp2leads_License::deactivate_license($license_email, $license_key);

        echo json_encode($result);
        die();
    }

    /**
     *  Ajax license removing
     */
    public function ajax_license_removing() {
        $license_email = isset($_POST['licenseEmail']) ? $_POST['licenseEmail'] : '';
        $license_key = isset($_POST['licenseKey']) ? $_POST['licenseKey'] : '';
        $site = isset($_POST['site']) ? $_POST['site'] : '';

        $result = Wp2leads_License::remove_license($license_email, $license_key, $site);

        echo json_encode($result);
        die();
    }

    /**
     *  Ajax complete activation process and encrypt license key
     */
    public function ajax_complete_activation() {
        $result = Wp2leads_License::complete_license_activation();

        echo json_encode($result);
        die();
    }

    public function save_results_to_transient() {
        $map_results = !empty($_POST['mapResults']) ? $_POST['mapResults'] : false;

        echo json_encode(array('success' => 1));
        die();
    }

    public function set_transient() {
        $transient_name = !empty($_POST['transient_name']) ? $_POST['transient_name'] : false;
        $transient_value = !empty($_POST['transient_value']) ? $_POST['transient_value'] : 1;

        set_transient($transient_name, $transient_value);
        $result = array('success' => 1);

        echo json_encode($result);
        die();
    }

    public function get_transient() {
        $transient_name = !empty($_POST['transient_name']) ? $_POST['transient_name'] : false;

        $value = get_transient($transient_name);
        $result = array('success' => 1, 'message' => $value);

        echo json_encode($result);
        die();
    }

    public function delete_transient() {
        $transient_name = !empty($_POST['transient_name']) ? $_POST['transient_name'] : false;

        delete_transient($transient_name);
        $result = array('success' => 1);

        echo json_encode($result);
        die();
    }

    /**
     *
     */
    public function ajax_get_global_table_search_results() {
        $table = sanitize_text_field($_POST['table']);
        $column = false;
        $order = false;

        if (!empty($_POST['column'])) {
            $column = sanitize_text_field($_POST['column']);
            $column = explode('(', $column);
            $column = trim($column[0]);
        }

        if (!empty($_POST['order'])) {
            $order = sanitize_text_field($_POST['order']);
        }

        echo json_encode(MapBuilderManager::get_table_search_results($table, $column, $order));

        wp_die();
    }

    /**
     *
     */
    public function ajax_get_global_multisearch_results() {
        $string = sanitize_text_field($_POST['string']);
        echo json_encode(MapBuilderManager::get_multisearch_results($string));

        wp_die();
    }

    public function ajax_get_single_multisearch_table() {
        $string = sanitize_text_field($_POST['string']);
        $table = sanitize_text_field($_POST['table']);
        $column = sanitize_text_field($_POST['column']);

        //echo json_encode(MapBuilderManager::get_multisearch_table($table, $column, $string));
        echo json_encode(MapBuilderManager::get_single_table($table, $string));

        wp_die();
    }
}