<?php
/**
 * Template for displaying Import/Export tab content
 *
 * This file is used to markup the admin-facing aspects of the plugin.
 *
 * @since      1.0.0
 *
 * @package    Wp2leads
 * @subpackage Wp2leads/admin/partials
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

$is_export_allowed = Wp2leads_License::is_action_allowed('export_own_map');
$maps_from_server = MapBuilderManager::get_available_maps_from_server();
$maps_pending_from_server = Wp2leads_License::is_dev_allowed() ? MapBuilderManager::get_pending_maps_from_server() : false;
$current_owner_hash = MapBuilderManager::get_current_map_owner_hash();
?>

<h2><?php _e('Maps on Server', 'wp2leads') ?></h2>

<hr>

<div class="import-maps-from-server-by-public-id__holder" style="width: 100%;max-width: 800px">
    <form id="import-map-from-server-by-public-id-form">
        <label for="">
            <h4><?php _e('Input Map Public ID to import', 'wp2leads') ?></h4>

            <input id="map-public-id" type="text" class="map-public-id form-control" value="">
        </label>

        <p class="submit">
            <button id="import_from_remote" type="submit" class="button button-primary">
                <?php _e('Import Map', 'wp2leads') ?>
            </button>
        </p>
    </form>
</div>
<?php
if ($maps_pending_from_server) {
    ?>
    <div class="import-pending-maps-from-server__section">
        <h3>Pending Maps</h3>

        <div class="import-pending-maps-from-server__holder" style="width: 100%;max-width: 800px">
            <form id="import-pending-maps-from-server-form">
                <div class="import-maps-from-server-table-wrap">
                    <table id="import-maps-from-server-table" class="thead-responsive-table" style="width: 100%;min-width: 700px">
                        <thead>
                        <tr>
                            <th></th>
                            <th><input type="checkbox" id="wp2l_select_all_import_maps" /></th>
                            <th><?php _e('Map', 'wp2leads') ?></th>
                            <th><?php _e('Map Version', 'wp2leads') ?></th>
                        </tr>
                        </thead>

                        <tbody>
                        <?php
                        foreach ($maps_pending_from_server as $map_from_server) {
                            ?>
                            <tr>
                                <td class="map-owner_column" style="width: 24px;text-align: center">
                                    <?php
                                    if ($current_owner_hash === $map_from_server['owner_hash']) {
                                        ?>
                                        <span class="dashicons dashicons-admin-users wp2lead-tip" data-tip="Ovid"></span>
                                        <?php
                                    }
                                    ?>
                                </td>
                                <td style="width: 30px;text-align: center">
                                    <input
                                            name="mapids[]"
                                            class="map-public-ids"
                                            type="checkbox"
                                            id="mappublicid_<?php echo $map_from_server['map_id'] ?>"
                                            value="<?php echo $map_from_server['map_id'] ?>"
                                            data-map_hash="<?php echo $map_from_server['map_hash'] ?>"
                                            data-owner_hash="<?php echo $map_from_server['owner_hash'] ?>"
                                    />
                                </td>

                                <td>
                                    <strong><?php echo $map_from_server['name']; ?></strong>
                                    <?php
                                    if (!empty($map_from_server['map_version'])) {
                                        ?>
                                        <small>v. <?php echo $map_from_server['map_version']; ?></small>
                                        <?php
                                    }

                                    ?>
                                </td>

                                <td style="text-align:center;">
                                    <?php
                                    $map_from_server_kind = $map_from_server['map_kind'];

                                    if ($map_from_server_kind == "pro") {
                                        _e('Pro', 'wp2leads');
                                    } elseif ($map_from_server_kind == "essent") {
                                        _e('Essential', 'wp2leads');
                                    } elseif ($map_from_server_kind == "free") {
                                        _e('Free', 'wp2leads');
                                    }
                                    ?>
                                </td>
                            </tr>
                            <?php
                        }
                        ?>
                        </tbody>
                    </table>
                </div>

                <p class="submit">
                    <button id="import_from_remote" type="submit" class="button button-primary">
                        <?php _e('Import from Map Server', 'wp2leads') ?>
                    </button>
                </p>
            </form>
        </div>
    </div>
    <?php
}


if (!$maps_from_server) {
    ?>
    <div class="wp2leads-notice wp2leads-notice-warning">
        <h4><?php _e('No available maps on server', 'wp2leads') ?></h4>
    </div>
    <?php
} else {
    ?>
    <div class="import-maps-from-server__holder" style="width: 100%;max-width: 800px">
        <form id="import-maps-from-server-form">
            <div class="import-maps-from-server-table-wrap">
                <table id="import-maps-from-server-table" class="thead-responsive-table" style="width: 100%;min-width: 700px">
                    <thead>
                    <tr>
                        <th>
                            <input type="checkbox" id="wp2l_select_all_import_maps" />
                        </th>
                        <th><?php _e('Map', 'wp2leads') ?></th>
                        <th colspan="3"><?php _e('Map Version', 'wp2leads') ?></th>
                    </tr>
                    </thead>

                    <tbody>
                    <?php
                    foreach ($maps_from_server as $map_from_server) {
                        $map_from_server_kind = $map_from_server['map_kind'];
                        $is_map_from_server_can_be_transfered = MapBuilderManager::is_map_on_server_can_be_transfered($wp2l_current_version, $map_from_server_kind);
                        ?>
                        <tr>
                            <td style="text-align: center">
                                <?php
                                if ('exclusive' !== $map_from_server_kind) {
                                    ?>
                                    <input
                                            name="mapids[]"
                                            class="map-public-ids"
                                            type="checkbox"
                                            id="mappublicid_<?php echo $map_from_server['map_id'] ?>"
                                            value="<?php echo $map_from_server['map_id'] ?>"
                                            data-map_hash="<?php echo $map_from_server['map_hash'] ?>"
                                            data-owner_hash="<?php echo $map_from_server['owner_hash'] ?>"
                                    />
                                    <?php
                                }
                                ?>
                            </td>

                            <td>
                                <strong><?php echo $map_from_server['name']; ?></strong>
                                <?php
                                if (!empty($map_from_server['map_version'])) {
                                    ?>
                                    <small>v. <?php echo $map_from_server['map_version']; ?></small>
                                    <?php

                                    if ('exclusive' === $map_from_server_kind) {
                                        ?><a href="https://wp2leads-for-klick-tipp.com/"><?php _e('Buy this map', 'wp2leads') ?></a><?php
                                    }
                                }

                                ?>
                            </td>

                            <td class="map-owner_column" style="text-align: center">
                                <?php
                                if ($current_owner_hash === $map_from_server['owner_hash']) {
                                    ?>
                                    <span class="dashicons dashicons-admin-users wp2lead-tip" data-mode="above" data-tip="<?php _e('Own map', 'wp2leads') ?>"></span>
                                    <?php
                                }
                                ?>
                            </td>

                            <td class="map-transfer-allowed_column" style="text-align: center">
                                <?php
                                if ( $is_map_from_server_can_be_transfered ) {
                                   ?>
                                    <span class="dashicons dashicons-unlock wp2lead-tip" data-mode="above" data-tip="<?php _e('Transfer allowed', 'wp2leads') ?>"></span>
                                    <?php
                                } else {
                                    ?>
                                    <span class="dashicons dashicons-lock wp2lead-tip" data-mode="above" data-tip="<?php _e('Transfer not allowed', 'wp2leads') ?>"></span>
                                    <?php
                                }
                                ?>
                            </td>

                            <td style="text-align:center;">
                                <?php
                                if ($map_from_server_kind == "pro") {
                                    _e('Pro', 'wp2leads');
                                } elseif ($map_from_server_kind == "essent") {
                                    _e('Essential', 'wp2leads');
                                } elseif ($map_from_server_kind == "exclusive") {
                                    _e('Exclusive', 'wp2leads');
                                } elseif ($map_from_server_kind == "free") {
                                    _e('Free', 'wp2leads');
                                }
                                ?>
                            </td>
                        </tr>
                        <?php
                    }
                    ?>
                    </tbody>
                </table>
            </div>

            <p class="submit">
                <button id="import_from_remote" type="submit" class="button button-primary">
                    <?php _e('Import Maps', 'wp2leads') ?>
                </button>
            </p>
        </form>
    </div>
    <?php
}
?>

<h2><?php _e('Local Maps', 'wp2leads') ?></h2>

<hr>

<?php if ($is_export_allowed) : ?>
    <h4><?php _e('Select the maps you would like to export:', 'wp2leads') ?></h4>
<?php else : ?>
    <div class="wp2leads-notice wp2leads-notice-warning">
        <h4><?php _e('Export Map only in Pro version', 'wp2leads') ?></h4>

        <p><?php _e('Please upgrade to Professional Version to export maps.', 'wp2leads') ?> <a href="https://wp2leads-for-klick-tipp.com/?p=7" target="_blank"><?php _e('Click here!', 'wp2leads') ?></a></p>
    </div>
<?php endif; ?>
<div class="export-maps-form__holder" style="width: 100%;max-width:800px">
<form id="export-maps-form" method="post">
    <input type="hidden" name="download_map_data" id="download_map_data">

    <div class="export-maps-to-server-table-wrap">
        <table id="export-maps_table" class="thead-responsive-table" style="width: 100%;min-width: 700px">
            <thead>
            <tr>

                <th>
                    <?php _e('Export', 'wp2leads') ?><br>
                    <input type="checkbox" id="wp2l_select_all_mapuploadids"<?php echo $is_export_allowed ? '' : ' disabled'; ?> />
                </th>

                <th>
                    <?php _e('Update', 'wp2leads') ?><br>
                    <input type="checkbox" id="wp2l_select_all_mapupdateids"<?php echo $is_export_allowed ? '' : ' disabled'; ?> />
                </th>

                <th><?php _e('Map', 'wp2leads') ?></th>

                <th colspan="4"><?php _e('Version (status)', 'wp2leads') ?></th>
            </tr>
            </thead>

            <tbody>
            <?php $i=0; foreach($maps as $map): ?>
                <?php
                $map_status = false;
                $decoded_map_port_info = unserialize($map['info']);
                $is_map_port_owner = MapBuilderManager::is_map_owner($decoded_map_port_info);
                $is_map_port_on_server = MapBuilderManager::is_map_on_server($decoded_map_port_info);
                $is_map_on_server_outdated = MapBuilderManager::is_map_on_server_outdated($map['id']);
                $map_port_kind = '';

                if ($is_map_port_on_server) {
                    $map_port_kind = !empty($decoded_map_port_info['publicMapKind']) ? $decoded_map_port_info['publicMapKind'] : false;
                }

                if ($is_map_port_owner && $is_map_port_on_server) {
                    $map_status = MapBuilderManager::check_map_status_on_server($map['id']);
                }
                ?>

                <tr>
                    <td style="width: 45px;max-width: 20%;text-align: center">
                        <?php
                        if ($is_map_port_owner && !$is_map_port_on_server) {
                            ?>
                            <input
                                    name="mapuploadids[<?php echo $map['id']?>]"
                                    class="map-upload"
                                    type="checkbox"
                                    id="mapkind_<?php echo $i ?>"
                                    value="<?php echo $map['id']?>"<?php echo $is_export_allowed ? '' : ' disabled'; ?>
                            />
                            <?php
                        }
                        ?>
                    </td>

                    <td style="width: 45px;max-width: 20%;text-align: center">
                        <?php
                        if ( $is_map_port_on_server && $is_map_port_owner && $is_map_on_server_outdated && 'pending' !== $map_status) {
                            ?>
                            <input
                                    name="mapupdateids[<?php echo $map['id']?>]"
                                    class="map-update"
                                    type="checkbox"
                                    id="mapkind_<?php echo $i ?>"
                                    value="<?php echo $map['id']?>"<?php echo $is_export_allowed ? '' : ' disabled'; ?>
                            />
                            <?php
                        }
                        ?>
                    </td>

                    <td>
                        <strong><?php echo $map['name']; ?></strong>
                        <?php
                        if ($is_map_port_on_server && !empty($decoded_map_port_info['publicMapVersion'])) {
                            echo ' <small>v. '. $decoded_map_port_info['publicMapVersion'] . '</small>';
                        }
                        ?>
                    </td>

                    <td class="map-owner_column" style="width: 18px;text-align: center">
                        <?php
                        if ($is_map_port_owner) {
                            ?>
                            <span class="dashicons dashicons-admin-users wp2lead-tip" data-mode="above" data-tip="<?php _e('Own map', 'wp2leads') ?>"></span>
                            <?php
                        }
                        ?>
                    </td>

                    <td class="map-on-server_column" style="width: 18px;text-align: center">
                        <?php
                        if ($is_map_port_on_server) {
                            ?>
                            <span class="dashicons dashicons-admin-site wp2lead-tip" data-mode="above" data-tip="<?php _e('Published', 'wp2leads') ?>"></span>
                            <?php
                        }
                        ?>
                    </td>

                    <td class="map-transfer-allowed_column" style="text-align: center">
                        <?php
                        if ( Wp2leads_License::is_map_transfer_allowed($map['id']) ) {
                            ?>
                            <span class="dashicons dashicons-unlock wp2lead-tip" data-mode="above" data-tip="<?php _e('Transfer allowed', 'wp2leads') ?>"></span>
                            <?php
                        } else {
                            ?>
                            <span class="dashicons dashicons-lock wp2lead-tip" data-mode="above" data-tip="<?php _e('Transfer not allowed', 'wp2leads') ?>"></span>
                            <?php
                        }
                        ?>
                    </td>

                    <td style="width: 80px;max-width: 25%;text-align: center">
                        <?php
                        if ($is_map_port_owner && !$is_map_port_on_server) {
                            ?>
                            <select name="mapkindversion[<?php echo $map['id']?>]" class="map-kind-version" id="mapkindversion_<?php echo $map['id']?>"<?php echo $is_export_allowed ? '' : ' disabled'; ?>>
                                <option value="pro" <?php echo ($map_port_kind == "pro" ? 'selected' : ''); ?>><?php _e('Pro', 'wp2leads') ?></option>
                                <?php
                                if ('pro' === $wp2l_current_version) {
                                    ?>
                                    <option value="exclusive" <?php echo ($map_port_kind == "exclusive" ? 'selected' : ''); ?>><?php _e('Exclusive', 'wp2leads') ?></option>
                                    <?php
                                }
                                ?>

                                <option value="essent" <?php echo ($map_port_kind == "essent" ? 'selected' : ''); ?>><?php _e('Essential', 'wp2leads') ?></option>
                                <option value="free" <?php echo ($map_port_kind == "free" ? 'selected' : ''); ?>><?php _e('Free', 'wp2leads') ?></option>
                            </select>
                            <?php
                        } else {
                            ?>
                            <input type="hidden" name="mapkindversion[<?php echo $map['id']?>]" class="map-kind-version" value="<?php echo $map_port_kind ?>" id="mapkindversion_<?php echo $map['id']?>" />
                            <?php
                            if ($map_port_kind == "pro") {
                                _e('Professional', 'wp2leads');
                            } elseif ($map_port_kind == "essent") {
                                _e('Essential', 'wp2leads');
                            } elseif ($map_port_kind == "free") {
                                _e('Free', 'wp2leads');
                            }

                            if ($map_status) {
                                echo ' <br>('. $map_status . ')';
                            }
                        }
                        ?>
                    </td>
                </tr>
                <?php $i++; endforeach; ?>

            </tbody>
        </table>
    </div>

    <?php
    if ($is_export_allowed) :
        submit_button( __('Export Maps', 'wp2leads') );
    else :
        submit_button( __('Export Maps', 'wp2leads'), 'secondary', '', true, array('disabled' => 'disabled') );
    endif;
    ?>
</form>
</div>