<?php
/**
 * Created by PhpStorm.
 * User: oleksii.khodakivskyi
 * Date: 19.08.18
 * Time: 12:19
 */

class Wp2LeadsCron {
    private static $cron_status_option_name = 'wp2leads_cron_maps';
    private $default_interval = 5 * 60;
    private $scheduled_maps = array();

    public function __construct() {
        $this->scheduled_maps = $this->getScheduledMaps();

        $this->addFilter();
        $this->addAction();
    }

    /**
     * Add Cron Filter
     *
     * @access private
     * @return void
     */
    private function addFilter() {
        add_filter('cron_schedules', array($this, 'cron_time_intervals'));
    }

    /**
     * Add the WP Cron Action
     *
     * @access private
     * @return void
     */
    private function addAction() {
        add_action('wp2leads_send_data_to_klick_tipp', array($this, 'sendData'));
    }

    /**
     * Set the schedule hooks
     *
     * @access public
     * @return void
     */
    public function setScheduleHook() {
        if (!wp_next_scheduled('wp2leads_send_data_to_klick_tipp')) {
            wp_schedule_event(time(), 'five_min', 'wp2leads_send_data_to_klick_tipp');
        }
    }

    /**
     * Clear the schedule hooks
     *
     * @access public
     * @return void
     */
    public function clearScheduleHook() {
        wp_clear_scheduled_hook('wp2leads_send_data_to_klick_tipp');
    }

    /**
     * Add new schedule for wordpress cron
     *
     * @access public
     * @return array
     */
    public function cron_time_intervals( $schedules ) {
        $schedules['five_min'] = array(
            'interval'=> $this->default_interval,
            'display'=> 'Every 5 minutes'
        );

        return $schedules;
    }

    public static function getScheduledMaps() {
        return json_decode(get_option(self::$cron_status_option_name), true);
    }

    public static function getScheduledMapsIds() {
        $sheduled_maps = array();
        $option_value = json_decode(get_option(self::$cron_status_option_name), true);

        foreach ($option_value as $key => $data) {
            $sheduled_maps[] = $data['id'];
        }

        return $sheduled_maps;
    }

    public function save_cron_status() {
        $response = array(
            'error' => 1,
            'success' => 0,
            'message' => 'Can\'t scheduled this map.'
        );

        $map_id = trim($_POST['map_id']);
        $key = 'map_'. $map_id;

        if (!empty($map_id)) {
            $existed_cron_maps = $this->getScheduledMaps();

            if (empty($existed_cron_maps)) {
                $existed_cron_maps = array();
            }

            if ( !empty( $_POST['map_id'] ) ) {
                $status = !empty( $_POST['cron_status'] ) ? true : false;
                $date_base = !empty( $_POST['date_base_for_cron'] ) ? $_POST['date_base_for_cron'] : array();
                $existed_cron_maps[$key] = array(
                    'id' => $map_id,
                    'status'    => $status,
                    'date_base' => $date_base
                );
            }

            if ( !empty( $_POST['cron_status'] ) ) {
                $message = 'Map with id ' . $map_id . ' was successfully scheduled by cron.';
            } else {
                $message = 'Map with id ' . $map_id . ' was successfully unscheduled by cron.';
            }

            $cron_status = json_encode($existed_cron_maps);
            $old_cron_status = get_option( self::$cron_status_option_name );

            if ( $cron_status === $old_cron_status ) {
                $result = true;
            } else {
                $result = update_option(self::$cron_status_option_name, $cron_status);
            }

            $cronChecked = isset($_POST['cron_status']) && 'checked' === $_POST['cron_status'] ? 'checked' : '';
            $cronSelected = json_encode( $_POST['date_base_for_cron'], JSON_HEX_QUOT );

            if ($result) {
                $response = array(
                    'error' => 0,
                    'success' => 1,
                    'message' => $message,
                    'cronChecked' => $cronChecked,
                    'cronSelected' => $cronSelected,
                );
            }
        }

        echo json_encode($response);
        wp_die();
    }

    /**
     * Execute plugin cron jobs
     *
     * Debug cron with the following url
     * http://[URL]/wp-cron.php?doing_cron
     *
     * @return void
     */
    public function sendData() {
        if(empty($this->getScheduledMaps())) {
            return;
        }

        foreach ($this->getScheduledMaps() as $key => $data) {
            $is_transfer_allowed = Wp2leads_License::is_map_transfer_allowed($data['id']);
            $map = MapsModel::get($data['id']);
            $decodedMap = unserialize($map->mapping);

            if (empty($decodedMap['dateTime'])) {
                $existed_cron_maps = $this->getScheduledMaps();
                unset($existed_cron_maps['map_' . $data['id']]);
                update_option(self::$cron_status_option_name, json_encode($existed_cron_maps));
            } elseif ( $is_transfer_allowed && $data['status']) {
                $date = time();
                $transfering_data = $this->prepareData($data);

                if ( empty( $transfering_data ) ) {
                    continue;
                }

                $total_transferred = 0;
                $result = KlickTippManager::transfer_data_to_kt( $data['id'], $transfering_data, 'cron' );

                if ( $result ) {
                    $new_subscribers_amount = $result['added_subscribers'];
                    $updated_subscribers_amount = $result['existed_subscribers'];
                    $total_transferred = $new_subscribers_amount + $updated_subscribers_amount;
                }

                $existed_cron_maps = $this->getScheduledMaps();

                if ( 0 < $total_transferred ) {

                    $existed_cron_maps['map_' . $data['id']]['last_send'] = $date;

                    update_option(self::$cron_status_option_name, json_encode($existed_cron_maps));
                }

                $existed_cron_maps['map_' . $data['id']]['last_check'] = $date;

                update_option(self::$cron_status_option_name, json_encode($existed_cron_maps));
            }
        }
    }

    private function prepareData($data) {

        $map = MapsModel::get($data['id']);
        $decodedMap = unserialize($map->mapping);
        $decodedApiSettings = unserialize($map->api);

        if ( !is_array( $data['date_base'] ) ) {
            $date_base = array( $data['date_base'] );
        } else {
            $date_base = $data['date_base'];
        }

        $last_cron = 0;

        if (!empty($data['last_check'])) {
            $last_cron = $data['last_check'];
        }

        $results = MapsModel::get_map_query_results($decodedMap, null, null);
        $results_to_send = array();

        foreach ($results as $key => $result) {
            foreach ($result as $column => $value) {
                if (in_array($column, $date_base) && !empty($value)) {
                    $date = ApiHelper::date_to_timestamp($value);

                    if ($date > $last_cron) {
                        $results_to_send[] = $results[$key];

                        break;
                    }
                }
            }
        }

        $tags_prefix = ApiHelper::get_map_tags_prefix($data['id']);
        $data_for_transfer = ApiHelper::prepareDataForTransfer($decodedApiSettings, $results_to_send, $tags_prefix);

        return $data_for_transfer;
    }

}