<?php
/**
 * Created by PhpStorm.
 * User: oleksii.khodakivskyi
 * Date: 30.07.18
 * Time: 23:19
 */

class ApiHelper {

    private static $minutes_number = 60;

    private static $hours_number = 24;

    private static $days_number = 31;

    private static $unique_subscribers_option_name = 'wp2l_unique_subscribers_counter';

    public static function terminate_map_to_api($map_id) {
        global $wpdb;

        $maptoapi_bg_in_process = get_transient('wp2leads_maptoapi_bg_in_process');

        if ($maptoapi_bg_in_process && !empty($maptoapi_bg_in_process[$map_id])) {

            $bg_total = 0;
            $bg_done = 0;
            $bg_count = 0;
            $bg_new = 0;
            $bg_updated = 0;

            foreach ($maptoapi_bg_in_process[$map_id] as $batch_key => $transfer_data) {
                $bg_total += $transfer_data['total'];
                $bg_done += $transfer_data['done'];
                $bg_count += $transfer_data['count'];
                $bg_new += $transfer_data['new'];
                $bg_updated += $transfer_data['updated'];

                $wpdb->query( "DELETE FROM $wpdb->options WHERE option_name LIKE '%wp_wp2leads_maptoapi_batch_".$batch_key."%'" );
            }


            unset($maptoapi_bg_in_process[$map_id]);

            $total_transferred = $bg_new + $bg_updated;

            if ($total_transferred > 0) {
                $data = array(
                    'time' => time(),
                    'map_id' => $map_id,
                    'transfer_type' => 'manually',
                    'statistics' => array (
                        'Total Amount' => $bg_total,
                        'New subscribers' => $bg_new,
                        'Updated subscribers' => $bg_updated,
                        'Total transferred' => $total_transferred
                    )
                );

                StatisticsManager::saveStatistics($data);
            }

            if (0 === count($maptoapi_bg_in_process)) {
                delete_transient('wp2leads_maptoapi_bg_in_process');
            } else {
                set_transient('wp2leads_maptoapi_bg_in_process', $maptoapi_bg_in_process);
            }
        }

        return true;
    }

    public static function get_paths($results, $decodedMap) {
        $paths = array();

        foreach ($results as $index => $result) {
            $result = (array) $result;

            foreach ($result as $table_column => $value) {
                if (false/*strpos($table_column, '(concatenated)') !== false*/) {
                    $parts = explode(', ', $value);

                    foreach ($parts as $part) {
                        $part = json_encode(trim($part), JSON_HEX_QUOT | JSON_HEX_TAG | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
                        $part = str_replace("\n", " ", str_replace('"', "", $part));
                        $paths[] = $table_column . ' (' . $part . ')';
                    }
                } else {
                    $value = json_encode(trim($value), JSON_HEX_QUOT | JSON_HEX_TAG | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
                    $value = str_replace("\n", " ", str_replace('"', "", $value));
                    $paths[] = $table_column . ' (' . $value . ')';
                }
            }
        }

        return $paths;
    }

    public static function get_options_paths($result, $decodedMap) {
        foreach ($result as $table_column => $value) {
            if (false/*strpos($table_column, '(concatenated)') !== false*/) {
                $parts = explode(', ', $value);

                foreach ($parts as $part) {
                    $part = json_encode(trim($part), JSON_HEX_QUOT | JSON_HEX_TAG | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
                    $part = str_replace("\n", " ", str_replace('"', "", $part));
                    $paths[] = $table_column . ' (' . $part . ')';
                }
            } else {
                $value = json_encode(trim($value), JSON_HEX_QUOT | JSON_HEX_TAG | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
                $value = str_replace("\n", " ", str_replace('"', "", $value));
                $paths[] = $table_column . ' (' . $value . ')';
            }
        }

        return $paths;
    }

    public static function date_to_timestamp($date_string) {
        $date = DateTime::createFromFormat('Y-m-d H:i:s', $date_string);

        if ($date) {
            return date_timestamp_get($date);
        }

        $date = DateTime::createFromFormat('Y-m-d', $date_string);

        if ($date) {
            return date_timestamp_get($date);
        }

        $date = DateTime::createFromFormat( 'U', (int) $date_string );

        if ($date) {
            return date_timestamp_get($date);
        }

        return false;
    }

    public static function arrayToString ($array, $glue = '.') {
        $paths = array();
        $index = 0;

        foreach ($array as $key => &$mixed) {
            if (is_array($mixed)) {
                $results = ApiHelper::arrayToString($mixed, $glue);

                foreach ($results as $k => &$v) {
                    $prep = '';

                    if (is_string($k) && is_bool($v)) {
                        if ($v) {
                            $v = $k;
                        } else {
                            $v = '';
                        }
                    } else {
                        if ( is_string( $k ) ) {
                            $prep = $k . ': ';
                        }

                        if ( ! is_string( $v ) && ! is_int( $v ) && ! is_float( $v ) && ! is_bool( $v ) ) {
                            $v = '';
                        }
                    }

                    $paths[$index] = $prep . (string) $v;
                    $index++;
                }
                unset($results);
            } else {
                $prep = '';

                if (is_string($key) && is_bool($mixed)) {
                    if ($mixed) {
                        $mixed = $key;
                    } else {
                        $mixed = '';
                    }
                } else {
                    if (is_string($key)) {
                        $prep = $key . ': ';
                    }

                    if (!is_string($mixed) && !is_int($mixed) && !is_float($mixed) && !is_bool($mixed)) {
                        $mixed = '';
                    }
                }

                $paths[$index] = $prep . (string) $mixed;
                $index++;
            }

        }

        return $paths;
    }

    public static function getSelectedOptin($conditions, $options, $default_optin) {
        $selected_option = $default_optin;

        if (!isset($conditions['optins']) || !is_array($conditions['optins'])) {
            return $selected_option;
        }

        foreach ($conditions['optins'] as $condition) {
            $value = $options->{$condition['option']};
            $condition_result = self::getConditionResult($value, $condition['string'], $condition['operator']);

            if ($condition_result) {
                $selected_option = $condition['connectTo'];
                break;
            }
        }

        return $selected_option;
    }

    private static function getConditionResult($value, $value_to, $operator) {
        $result = false;
        $value = trim($value);
        $value_to = trim($value_to);

        switch ($operator) {
            case 'is like':
                if ($value === $value_to) {
                    $result = true;
                }
                break;
            case 'like':
                if ($value === $value_to) {
                    $result = true;
                }
                break;
            case 'not-like':
                if ($value !== $value_to) {
                    $result = true;
                }
                break;
            case 'contains':
                if (strpos($value, $value_to) !== false) {
                    $result = true;
                }
                break;
            case 'not contains':
                if (strpos($value, $value_to) === false) {
                    $result = true;
                }
                break;
            case 'bigger as':
                if ((float) $value > (float)$value_to) {
                    $result = true;
                }
                break;
            case 'smaller as':
                if ((float)$value < (float)$value_to) {
                    $result = true;
                }
                break;
        }

        return $result;
    }

    public static function getUniqueTableColumns($paths) {
        $unique_array = array();

        foreach ($paths as $key => $value) {
            list($table_column, $current_option) = explode(' (', $value);

            if (!in_array($table_column, $unique_array) && "" !== trim(str_replace(')', '', $current_option))) {
                $unique_array[] = $table_column;
            }
        }

        return $unique_array;
    }

    public static function secureLicenseKey($license_key) {
        $parts = explode('-', $license_key);

        foreach ($parts as $index => $part) {
            if ($index >= 2 && $index <= 5) {
                $parts[$index] = 'XXXXX';
            }
        }

        return implode('-', $parts);
    }

    public static function get_map_tags_prefix($map_id) {
        $prefix = '';

        $map = MapsModel::get( $map_id );
        $decodedApiSettings = unserialize($map->api);

        $global_tag_prefix = get_option('wp2l_klicktipp_tag_prefix');

        if (!empty($decodedApiSettings['tags_prefix'])) {
            $prefix = $decodedApiSettings['tags_prefix'];
        } elseif (!empty($global_tag_prefix)) {
            $prefix = $global_tag_prefix;
        }

        return $prefix;
    }

    public static function prepareDataForTransfer($api, $data, $prefix = '') {
        $data_for_transfer = array();

        if (!empty($api['fields']['api_email']['table_columns'])) {
            $api_email_filed_name = $api['fields']['api_email']['table_columns'];

            foreach ($data as $item) {
                foreach ($api_email_filed_name as $tablecolumn) {
                    $email_column_value = trim($item->{$tablecolumn});
                    $email_column_valid = filter_var($email_column_value, FILTER_VALIDATE_EMAIL);

                    if ($email_column_valid) {
                        $user_email = $email_column_valid;
                        break;
                    }
                }

                if (empty($user_email)) {
                    continue;
                }

                $user = isset($data_for_transfer[$user_email]) ? $data_for_transfer[$user_email] : array();
                $user['optin'] = self::getSelectedOptin($api['conditions'], $item, $api['default_optin']);
                $user['fields'] = array();

                foreach ($api['fields'] as $key => $field_data) {
                    $api_key = str_replace('api_', '', $key);

                    if ('email' !== $api_key) {
                        $type = isset($field_data['type']) ? $field_data['type'] : '';

                        if (!empty($field_data['table_columns'])) {
                            foreach ($field_data['table_columns'] as $table_column) {
                                if (!empty($item->{$table_column})) {
                                    $user['fields'][$api_key][] = self::sanitizeValueByType($item->{$table_column}, $type);
                                }
                            }

                            $user['fields'][$api_key] = implode('; ', $user['fields'][$api_key]);
                        }
                    }
                }

                $user['tags'] = isset($user['tags']) && is_array($user['tags']) ? $user['tags'] : array();
                $tags = self::getTags($api['conditions']['tags'], $api['connected_for_tags'], $item, $prefix);
                $condition_tags = array_values(self::getConditionTags($api['conditions']['tags'], $item));
                $user['tags'] = array_unique(array_merge($user['tags'], $tags));

                $user['tags'] = array_filter(array_map('trim', $user['tags']), function($value) { return $value !== ''; });

                if (!empty($api['manually_selected_tags']['tag_ids'])) {
                    $user['manually_tags'] = $api['manually_selected_tags']['tag_ids'];
                } else {
                    $user['manually_tags'] = array();
                }

                $user['manually_tags'] = array_unique(array_merge($user['manually_tags'], $condition_tags));

                $user['detach_tags'] = isset($user['detach_tags']) && is_array($user['detach_tags']) ? $user['detach_tags'] : array();

                if (!empty($api['conditions']['detach_tags'])) {
                    $detach_tags = self::getTags($api['conditions']['detach_tags'], false, $item, '', 'detach');
                    $user['detach_tags'] = array_unique(array_merge($user['detach_tags'], $detach_tags));
                }

                $data_for_transfer[$user_email] = $user;
            }
        }

        return $data_for_transfer;
    }

    public static function getCurrentUserData($users_data, $email) {
        return array($email => $users_data[$email]);
    }

    private static function getConditionTags($tags_conditions, $data) {
        $tags = array();

        if (!empty($tags_conditions)) {
            foreach ($tags_conditions as $condition) {
                $condition_result = self::getConditionResult($data->{$condition['option']}, $condition['string'], $condition['operator']);

                if ($condition_result) {
                    $tags[] = $condition['connectTo'];
                }
            }
        }

        return array_map('trim', $tags);
    }

    private static function getTags($tags_conditions, $tags_connected_conditions, $data, $prefix = '', $type = false) {
        $tags = array();

        if (!empty($tags_conditions) && 'detach' === $type) {
            foreach ($tags_conditions as $condition) {
                $condition_result = self::getConditionResult($data->{$condition['option']}, $condition['string'], $condition['operator']);

                if ($condition_result) {
                    $separators = self::getSeparators($condition['option'], $tags_connected_conditions['separators']);

                    if (!empty($separators) && isset($data->{$condition['option']})) {
                        $temp_tags = explode($separators[0], str_replace($separators, $separators[0], $data->{$condition['option']}));
                        $tags = array_unique(array_merge($tags, $temp_tags));
                    } else {
                        $tags[] = $condition['connectTo'];
                    }
                }
            }
        }

        if (!empty($tags_connected_conditions['separators'])) {
            foreach ($tags_connected_conditions['separators'] as $separator_options) {
                if (!empty($separator_options['separator']) && !empty($separator_options['option'])) {
                    foreach ($separator_options['option'] as $table_column) {
                        $tags_concat = $data->{$table_column};
                        $tags_array_temp = explode($separator_options['separator'], $tags_concat);
                        $tags_array = array();

                        foreach ($tags_array_temp as $tag_temp) {
                            if (!empty(trim($tag_temp))) {
                                $tags_array[] = $prefix . ' ' . trim($tag_temp);
                            }
                        }

                        $tags = array_unique(array_merge($tags, $tags_array));
                    }
                }
            }
        }

        if (!empty($tags_connected_conditions['tags'])) {
            foreach ($tags_connected_conditions['tags'] as $table_column) {

                if (!empty(trim($data->{$table_column}))) {
                    $tags[] = $prefix . ' ' . trim($data->{$table_column});
                }
            }
        }

        if (!empty($tags_connected_conditions['tags_concat'])) {
            foreach ($tags_connected_conditions['tags_concat'] as $table_column) {
                $tags_concat = $data->{$table_column};
                $tags_array_temp = explode(',', $tags_concat);
                $tags_array = array();

                foreach ($tags_array_temp as $tag_temp) {
                    if (!empty(trim($tag_temp))) {
                        $tags_array[] = $prefix . ' ' . trim($tag_temp);
                    }
                }

                $tags = array_unique(array_merge($tags, $tags_array));
            }
        }

        return array_map('trim', $tags);
    }

    /**
     * @param $tags_conditions
     * @param $tags_connected_conditions
     * @param $data
     * @param bool $type
     *
     * @return array
     */
    private static function getTagsOld($tags_conditions, $tags_connected_conditions, $data, $type = false) {
        $tags = array();

        if (!empty($tags_conditions)) {
            foreach ($tags_conditions as $condition) {
                $condition_result = self::getConditionResult($data->{$condition['option']}, $condition['string'], $condition['operator']);

                if ($condition_result) {
                    $separators = self::getSeparators($condition['option'], $tags_connected_conditions['separators']);

                    if (!empty($separators) && isset($data->{$condition['option']})) {
                        $temp_tags = explode($separators[0], str_replace($separators, $separators[0], $data->{$condition['option']}));
                        $tags = array_unique(array_merge($tags, $temp_tags));
                    } else {
                        if ('detach' === $type){
                            $tags[] = $condition['connectTo'];
                        } else {
                            $tags[] = $data->{$condition['option']};
                        }
                    }
                }
            }
        }

        if (!empty($tags_connected_conditions['tags'])) {
            foreach ($tags_connected_conditions['tags'] as $table_column) {
                if (!empty($tags_connected_conditions['separators'])) {
                    $separators = self::getSeparators($table_column, $tags_connected_conditions['separators']);

                    if (!empty($separators) && isset($data->{$table_column})) {
                        $temp_tags = explode($separators[0], str_replace($separators, $separators[0], $data->{$table_column}));
                        $tags = array_unique(array_merge($tags, $temp_tags));
                    } else {
                        $tags[] = $data->{$table_column};
                    }
                } else {
                    $tags[] = $data->{$table_column};
                }
            }
        }

        return array_map('trim', $tags);
    }

    private static function getSeparators($table_column, $conditions) {
        $separators = array();

        if (!empty($conditions)) {
            foreach ($conditions as $separator) {
                if (in_array($table_column, $separator['option'])) {
                    $separators[] = $separator['separator'];
                }
            }
        }

        return $separators;
    }

    public static function getTagsIds($tag_names, $available_tags, $connector) {
        $ids = array();

        foreach ($tag_names as $tag_name) {

            $tag_name = str_replace('&#38;', '+', $tag_name);
            $tag_name = str_replace('&amp;', '+', $tag_name);
            $tag_name = str_replace('&', '+', $tag_name);

            $key = array_search(strtolower($tag_name), array_map('strtolower', $available_tags));

            if ($key !== false) {
                $ids[] = trim($key);
            } else {
                $tag_id = $connector->tag_create($tag_name);

                if ($tag_id) {
                    $ids[] = $tag_id;
                }
            }
        }

        return $ids;
    }

    private static function sanitizeValueByType($value, $type) {
        $sanitized_value = '';
        $value = trim($value);

        switch ($type) {
            case 'text':
                $sanitized_value = trim($value);
                break;
            case 'number':
                $sanitized_value = floatval($value);
                break;
            case 'date':
            case 'time':
                $time_zone = get_option('timezone_string');
                $utc_time_zone = 'UTC';

                $date = DateTime::createFromFormat( 'Y-m-d H:i:s', $value, new DateTimeZone($utc_time_zone) );

                if (!$date) {
                    $date = DateTime::createFromFormat( 'Y-m-d', $value, new DateTimeZone($utc_time_zone) );
                }

                if (!$date) {
                    $date = DateTime::createFromFormat( 'U', (int) $value, new DateTimeZone($utc_time_zone) );
                }

                if ($date) {
                    $sanitized_value = date_timestamp_get($date);
                } else {
                    $sanitized_value = '';
                }
                break;
            case 'url':
                $sanitized_value = $value;
                break;
        }

        return $sanitized_value;
    }

    public static function getMinutes() {
        $minutes = array();

        for ($i = 1; $i < self::$minutes_number; $i++) {
            $minutes[] = $i;
        }

        return $minutes;
    }

    public static function getHours() {
        $hours = array();

        for ($i = 1; $i < self::$hours_number; $i++) {
            $hours[] = $i;
        }

        return $hours;
    }

    public static function getDays() {
        $days = array();

        for ($i = 1; $i <= self::$days_number; $i++) {
            $days[] = $i;
        }

        return $days;
    }

    public static function getWeekdays() {
        $weekdays = array(
            'Monday',
            'Tuesday',
            'Wednesday',
            'Thusday',
            'Friday',
            'Saturday',
            'Sunday'
        );

        return $weekdays;
    }

    public static function getMonths() {
        $months = array(
            'January',
            'February',
            'March',
            'April',
            'May',
            'June',
            'July',
            'August',
            'September',
            'October',
            'November',
            'December'
        );

        return $months;
    }

    public static function getWoocommerceOrderStatuses() {
        $statuses = array();

        if (function_exists('wc_get_order_statuses')) {
            $statuses = array_keys(wc_get_order_statuses());
        }

        return $statuses;
    }

    public static function getDetachTags($map_id) {
        global $wpdb;
        $table = $wpdb->get_row(sprintf("SELECT * FROM %s WHERE id=%d", $wpdb->prefix . "wp2l_maps",(int)$map_id) );
        $api = unserialize($table->api);
        $tags = array();

        if (isset($api['detach_tags']['tag_ids'])) {
            $tags = array_merge($tags, $api['detach_tags']['tag_ids']);
        }

        return $tags;
    }

    public static function isValidTimeStamp($timestamp)  {
        return ((string) (int) $timestamp === $timestamp)
            && ($timestamp <= PHP_INT_MAX)
            && ($timestamp >= ~PHP_INT_MAX);
    }

    public static function convertTimeToLocal($time) {
        $format = 'Y-m-d H:i:s';
        $time_zone = get_option('timezone_string');
        $dt = new DateTime();
        $dt->setTimestamp($time);
        $dt->setTimezone(new DateTimeZone($time_zone));
        return $dt->format($format);
    }

    public static function setSubscribersCounter($map_id, $values) {
        $counter = array(
            'unique' => !empty($values['unique']) ? $values['unique'] : 0
        );

        $all_counters = json_decode(get_option(self::$unique_subscribers_option_name), true);

        if (!empty($all_counters[$map_id])) {
            foreach ($all_counters[$map_id] as $key => $value) {
                $all_counters[$map_id][$key] = $value + $counter[$key];
            }
        } else {
            $all_counters[$map_id] = $counter;
        }

        update_option(self::$unique_subscribers_option_name, json_encode($all_counters));
    }

    public static function getSubscribersCounter($map_id) {
        $counter = array(
            'unique' => 0
        );

        $all_counters = json_decode(get_option(self::$unique_subscribers_option_name), true);

        if (!empty($all_counters[$map_id])) {
            $counter = $all_counters[$map_id];
        }

        return $counter;
    }
}