<?php
/**
 * Class KlickTippManager
 */
class KlickTippManager {

    /**
     * @param $map_id
     * @param $data_for_transfer
     * @param string $type
     *
     * @return array|bool
     */
    public static function transfer_data_to_kt( $map_id, $data_for_transfer, $type = 'manually' ) {
        $connector = new Wp2leads_KlicktippConnector();
        $logged_in = $connector->login(get_option('wp2l_klicktipp_username'), get_option('wp2l_klicktipp_password'));

        if( $logged_in ) {
            $available_tags = $connector->tag_index();
            $detach_tags = ApiHelper::getDetachTags($map_id);

            $added_subscribers = array();
            $existed_subscribers = array();

            foreach ($data_for_transfer as $email => $data) {
                $tags = ApiHelper::getTagsIds($data['tags'], $available_tags, $connector);

                $result = self::transfer_subscriber_to_kt($connector, $email, $data, $tags, $detach_tags);

                if ( $result ) {
                    if ($result['added_subscriber']) {
                        $added_subscribers[] = $result['subscriber'];
                    } else if ($result['existed_subscriber']) {
                        $existed_subscribers[] = $result['subscriber'];
                    }
                }
            }

            $new_subscribers_amount = count($added_subscribers);
            $updated_subscribers_amount = count($existed_subscribers);
            $total_transferred = $new_subscribers_amount + $updated_subscribers_amount;

            $last_transferred_time = time();

            // $available_users = count($data_for_transfer) - ApiHelper::getSubscribersCounter($map_id)['unique'];
            $available_users = count($data_for_transfer);

            $counters = array(
                'unique' => $new_subscribers_amount
            );

            ApiHelper::setSubscribersCounter($map_id, $counters);

             if ($total_transferred > 0) {
                $data = array(
                    'time' => $last_transferred_time,
                    'map_id' => $map_id,
                    'transfer_type' => $type,
                    'statistics' => array(
                        'Total Amount' => $available_users > 0 ? $available_users : 0,
                        'New subscribers' => $new_subscribers_amount,
                        'Updated subscribers' => $updated_subscribers_amount,
                        'Total transferred' => $total_transferred
                    )
                );

                StatisticsManager::saveStatistics($data);
             }

            return array(
                'last_transferred_time' => $last_transferred_time,
                'added_subscribers' => $new_subscribers_amount,
                'existed_subscribers' => $updated_subscribers_amount
            );

        } else {
            return false;
        }
    }

    /**
     * @param Wp2leads_KlicktippConnector $connector
     * @param $email
     * @param $data
     * @param $tags
     * @param $detach_tags
     *
     * @return array
     */
    public static function transfer_subscriber_to_kt( Wp2leads_KlicktippConnector $connector, $email, $data, $tags, $detach_tags) {
        $subscriber_exist = $connector->subscriber_search($email);

        if (!empty($data['manually_tags'])) {
            $tags = array_merge($tags, $data['manually_tags']);
        }

        $detach_tags = array_merge($detach_tags, $data['detach_tags']);

        foreach ($tags as $key => $tag_id) {
            if (in_array($tag_id, $detach_tags)) {
                unset($tags[$key]);
            }
        }

        array_values($tags);

        // Add new Subscriber
        if ( !$subscriber_exist ) {
            $subscriber = $connector->subscribe($email, $data['optin'], 0, $data['fields'], '');

            if ($subscriber) {
                foreach ($tags as $tag_id) {
                    $connector->tag($email, $tag_id);
                }

                return array( 'subscriber' => $subscriber, 'added_subscriber' => true, 'existed_subscriber' => false );
            }
        }

        // Maybe update Subscriber
        else {
            $subscriber_changed = false;
            $subscriber = (array) $connector->subscriber_get($subscriber_exist);

            foreach ($data['fields'] as $code => $field_value) {
                if (isset($subscriber[$code])) {
                    if ($subscriber[$code] !== $field_value) {
                        $subscriber_changed = $connector->subscriber_update($subscriber_exist, $data['fields']);
                        break;
                    }
                }
            }

            foreach ($detach_tags as $tag_id) {
                if ( in_array( $tag_id, $subscriber['tags'] ) ) {
                    $response = $connector->untag($email, $tag_id);

                    if ($response) {
                        $subscriber_changed = true;
                    }
                }
            }

            foreach ($tags as $tag_id) {
                if (!in_array($tag_id, $subscriber['tags'])) {
                    $response = $connector->tag($email, $tag_id);

                    if ($response) {
                        $subscriber_changed = true;
                    }
                }
            }

            if ( $subscriber_changed ) {
                return array( 'subscriber' => $subscriber, 'added_subscriber' => false, 'existed_subscriber' => true );
            }

            return array( 'subscriber' => $subscriber, 'added_subscriber' => false, 'existed_subscriber' => false );
        }
    }
}