<?php
/**
 * Created by PhpStorm.
 * User: snth
 * Date: 27.10.18
 * Time: 20:03
 */

class MapBuilderManager {

    public static $excluded_maps = array(
        'wp2l_maps', 'wp2l_klicktipp', 'wp2leads_transfer_statistics', 'wptkt_log'
    );

    public static function get_map_query_results_limit() {
        $system_info = SystemHelper::get_system_info();
        $php_memory_limit = $system_info['php_memory_limit'];

        if (268435456 <= $php_memory_limit) {
            return 7500;
        } elseif(234881024 <= $php_memory_limit) {
            return 6500;
        } elseif(201326592 <= $php_memory_limit) {
            return 5500;
        } elseif(167772160 <= $php_memory_limit) {
            return 4500;
        } elseif(134217728 <= $php_memory_limit) {
            return 3500;
        } elseif(100663296 <= $php_memory_limit) {
            return 2500;
        } elseif (67108864 <= $php_memory_limit) {
            return 1500;
        }

        return false;
    }

    /**
     * Get table results
     *
     * @param $table
     * @return mixed
     */
    public static function get_table_search_results($table, $column = false, $order = false) {
        global $wpdb;

        $columns = MapBuilderManager::fetch_columns_for_table($table);

        $temp_columns = $wpdb->get_results('DESCRIBE ' . $wpdb->prefix . MapBuilderManager::unindexedTableName($table) . ';');

        $columns_titles = array_map(function ($item) {
            return $item->Field . ' ('.strtoupper(str_replace(' unsigned', '', $item->Type)).')';
        }, $wpdb->get_results('DESCRIBE ' . $wpdb->prefix . MapBuilderManager::unindexedTableName($table) . ';'));

        $columns_titles = array_values($columns_titles);

        $table = $wpdb->prefix.$table;

        $orderColumn = !empty($column) ? $column : $columns[0];
        $order = !empty($order) ? strtoupper($order) : 'DESC';

        $query = $wpdb->prepare(
            "SELECT * FROM {$table} ORDER BY {$orderColumn} {$order} LIMIT %d",
            50
        );

        $results = $wpdb->get_results($query, ARRAY_A);

        return array('columns' => $columns, 'results' => $results, 'columnsTitles' => $columns_titles);
    }

    /**
     * Results for multisearch
     *
     * @param $string
     * @return mixed
     */
    public static function get_multisearch_results($string) {
        global $wpdb;

        $tables = MapBuilderManager::fetch_tables();

        $enriched_tables = array_map(function ($table) use ($wpdb) {
            return [
                'table' => $wpdb->prefix . $table,
                'columns' => MapBuilderManager::fetch_columns_for_table($table)
            ];
        }, $tables);

        $results = array();

        foreach ($enriched_tables as $enriched_table) {

            $table_title = str_replace($wpdb->prefix, '', $enriched_table['table']);
            $table_title_group = false;

            $total = 0;
            $table_title_array = explode('_', $table_title);
            if (count($table_title_array) > 1) {
                $table_title_group = $table_title_array[0];
            }

            foreach ($enriched_table['columns'] as $column) {
                $result_count = $wpdb->get_var($wpdb->prepare(
                    "SELECT COUNT(*) FROM `{$enriched_table['table']}` WHERE `{$column}` LIKE '%s'",
                    '%' . $wpdb->esc_like($string) . '%'
                ));

                if ($result_count) {
                    $results[$table_title]['columns'][] = array(
                        'column' => $column,
                        'count' => $result_count
                    );

                    $total = $total + $result_count;

                    $results[$table_title]['total'] = $total;

                    if($table_title_group) {
                        $results[$table_title]['group'] = $table_title_group;
                    }
                }
            }
        }

        uasort($results, function($a,$b) {
            return ($a["total"] <= $b["total"]) ? 1 : -1;
        });

        return $results;
    }

    public static function contains_valid_json_maps($json)
    {
        foreach($json as $map) {
            if(!property_exists($map, 'name') || !property_exists($map, 'mapping')) {
                return false;
            }
        }
        return true;
    }

    public static function ingest_uploaded_json_maps($json)
    {
        global $wpdb;

        foreach ($json as $map) {
            $wpdb->insert(MapsModel::get_table(), [
                'name' => $map->name,
                'mapping' => $map->mapping,
                'time' => $map->time ? $map->time : date('Y-m-d H:i:s')
            ]);
        }
    }

    public static function get_multisearch_table($table, $column, $string) {
        global $wpdb;

        $columns = MapBuilderManager::fetch_columns_for_table($table);
        $table = $wpdb->prefix.$table;

        $query = $wpdb->prepare(
            "SELECT * FROM {$table} WHERE {$column} LIKE '%s'",
            '%' . $wpdb->esc_like($string) . '%'
        );

        $results = $wpdb->get_results($query, ARRAY_A);

        foreach ($results as $key => $result) {
            $column_value = $result[$column];
            $highlighted_value = str_replace($string, '<span class="highlighted">' . $string . '</span>', $column_value);
            $results[$key][$column] = $highlighted_value;
        }

        return array('columns' => $columns, 'results' => $results);
    }

    public static function get_single_table($table, $string) {
        global $wpdb;

        $columns = MapBuilderManager::fetch_columns_for_table($table);

        $columns_titles = array_map(function ($item) {
            return $item->Field . ' ('.strtoupper(str_replace(' unsigned', '', $item->Type)).')';
        }, $wpdb->get_results('DESCRIBE ' . $wpdb->prefix . MapBuilderManager::unindexedTableName($table) . ';'));

        $columns_titles = array_values($columns_titles);

        $table = $wpdb->prefix.$table;

        $query = "SELECT * FROM {$table} ";
        $i = 0;
        $placeholders = array();

        foreach ($columns as $column) {
            if ($i === 0) {
                $query .= " WHERE `{$column}` LIKE '%s'";

                $i++;
            } else {
                $query .= " OR `{$column}` LIKE '%s'";
            }

            $placeholders[] = '%' . $wpdb->esc_like($string) . '%';
        }

        $query .= ' LIMIT 1000';

        $query = $wpdb->prepare(
            $query,
            $placeholders
        );

        $results = $wpdb->get_results($query, ARRAY_A);

        return array('columns' => $columns_titles, 'results' => $results);
        // return array('columns' => $columns, 'results' => $results);
    }

    public static function fetch_tables() {
        global $wpdb;

        $tables = array_map(function ($item) use ($wpdb) {
            return str_replace($wpdb->prefix, '', array_values(get_object_vars($item))[0]);
        }, $wpdb->get_results('SHOW TABLES;'));

        $tables = array_values(array_diff($tables, MapBuilderManager::$excluded_maps));

        return $tables;
    }

    public static function fetch_columns_for_table($table)
    {
        global $wpdb;

        $table = MapBuilderManager::unindexedTableName($table);

        return array_map(function ($item) {
            return $item->Field;
        }, $wpdb->get_results('DESCRIBE ' . $wpdb->prefix . $table . ';'));
    }

    public static function unindexedTableName($table)
    {
        @list($table, $index) = explode('-', $table);
        return $table;
    }

    public static function is_map_owner($map_info) {
        $site_url = Wp2leads_License::get_current_site();

        $map_owner_hash = MapBuilderManager::get_current_map_owner_hash();

        if (!empty($map_info['publicMapOwner'])) {
            if ($map_info['publicMapOwner'] === $map_owner_hash) {
                return true;
            }
        } elseif (!empty($map_info['domain']) && $map_info['domain'] === $site_url) {
            return true;
        }

        return false;
    }

    public static function get_current_map_owner_hash() {
        $license_info = Wp2leads_License::get_lecense_info();
        $license_email = $license_info['email'];
        $license_key = $license_info['key'];
        $site_url = Wp2leads_License::get_current_site();

        $map_owner_hash = md5($site_url . $license_email . $license_key);

        return $map_owner_hash;
    }

    public static function is_map_on_server($map_info) {
        if (!empty($map_info['publicMapId']) && !empty($map_info['publicMapOwner']) && !empty($map_info['publicMapStatus'])) {
            return true;
        }

        return false;
    }

    public static function is_map_on_server_can_be_transfered($license_version, $map_kind) {
        $license_level = array(
            'pro'       => 3,
            'exclusive'       => 3,
            'essent'    => 2,
            'free'      => 1
        );

        return $license_level[$license_version] >= $license_level[$map_kind];
    }

    public static function is_map_on_server_outdated($map_id) {
        $map = MapsModel::get($map_id);
        $map_info = unserialize($map->info);

        if (MapBuilderManager::is_map_on_server($map_info)) {
            $server_map_content_hash = $map_info['publicMapContent'];
            $current_map_content_hash = MapBuilderManager::get_map_content_hash($map);

            if ($server_map_content_hash !== $current_map_content_hash) {
                return true;
            }
        }

        return false;
    }

    public static function get_map_content_hash($map) {
        $mapping = $map->mapping;
        $api = $map->api;

        return md5($mapping . $api);
    }

    public static function get_map_hash($mapping) {
        $map_from = !empty($mapping['from']) ? serialize($mapping['from']) : '';
        $map_joins = !empty($mapping['joins']) ? serialize($mapping['joins']) : '';
        $map_groupConcat = !empty($mapping['groupConcat']) ? serialize($mapping['groupConcat']) : '';
        $map_virtualRelationships = !empty($mapping['virtual_relationships']) ? serialize($mapping['virtual_relationships']) : '';
        $map_keyBy = !empty($mapping['keyBy']) ? serialize($mapping['keyBy']) : '';

        return md5($map_from . $map_joins . $map_groupConcat . $map_virtualRelationships . $map_keyBy);
    }

    public static function get_map_on_server_kind() {

    }

    public static function import_maps_from_server($maps) {
        global $wpdb;

        $table = $wpdb->prefix . 'wp2l_maps';
        $license_info = Wp2leads_License::get_lecense_info();
        $license_email = $license_info['email'];
        $license_key = $license_info['key'];
        $site_url = Wp2leads_License::get_current_site();

        $parameters = array (
            'license_email' => $license_email,
            'license_key' => $license_key,
            'site_url' => $site_url,
            'event' => 'import',
            'maps'  =>  serialize($maps)
        );

        $request = wp_remote_post(
            base64_decode('aHR0cHM6Ly9tYXBzLndwMmxlYWRzLmNvbS9zZXJ2ZXIvbWFwcy5waHA='),
            array(
                'body'    => $parameters,
            )
        );

        $response = json_decode(wp_remote_retrieve_body( $request ), true);

        if (200 === $response['code']) {
            $maps = $response['body']['maps'];
            $count = count($maps);

            if ($count === 0) {
                return false;
            }

            foreach ($maps as $map) {
                $name = $map['name'];
                $mapping = $map['mapping'];
                $api = $map['api'];
                $info = $map['info'];

                $wpdb->insert($table, [
                    'name' => $name,
                    'mapping' => $mapping,
                    'api' => $api,
                    'info' => $info,
                    'time' => date('Y-m-d H:i:s')
                ]);
            }

            return $count;
        }

        return false;

    }

    public static function import_pending_maps_from_server($maps) {
        global $wpdb;

        $table = $wpdb->prefix . 'wp2l_maps';
        $license_info = Wp2leads_License::get_lecense_info();
        $license_email = $license_info['email'];
        $license_key = $license_info['key'];
        $site_url = Wp2leads_License::get_current_site();

        $parameters = array (
            'license_email' => $license_email,
            'license_key' => $license_key,
            'site_url' => $site_url,
            'event' => 'import_pending',
            'maps'  =>  serialize($maps)
        );

        $request = wp_remote_post(
            base64_decode('aHR0cHM6Ly9tYXBzLndwMmxlYWRzLmNvbS9zZXJ2ZXIvbWFwcy5waHA='),
            array(
                'body'    => $parameters,
            )
        );

        $response = json_decode(wp_remote_retrieve_body( $request ), true);

        if (200 === $response['code']) {
            $maps = $response['body']['maps'];
            $count = count($maps);

            if ($count === 0) {
                return false;
            }

            foreach ($maps as $map) {
                $name = $map['name'];
                $mapping = $map['mapping'];
                $api = $map['api'];
                $info = $map['info'];

                $wpdb->insert($table, [
                    'name' => $name,
                    'mapping' => $mapping,
                    'api' => $api,
                    'info' => $info,
                    'time' => date('Y-m-d H:i:s')
                ]);
            }

            return $count;
        }

        return false;

    }

    public static function export_maps_to_server($maps) {
        global $wpdb;

        $table = $wpdb->prefix . 'wp2l_maps';
        $maps_to_export_count = count($maps);
        $maps_ids = array();
        $maps_version = array();

        foreach ($maps as $map) {
            $maps_ids[] = $map['mapId'];
            $maps_version[$map['mapId']] = $map['mapVersion'];
        }

        $string_placeholders = array_fill(0, $maps_to_export_count, '%d');
        $placeholders_for_maps_id = implode(', ', $string_placeholders);

        $all_maps = $wpdb->get_results($wpdb->prepare(
            "SELECT * FROM " . $table . " WHERE id IN ($placeholders_for_maps_id)",
            $maps_ids
        ));

        foreach ($all_maps as $key => $map_obj) {
            $id = (int) $map_obj->id;
            $map_version = !empty($maps_version[$id]) ? $maps_version[$id] : '';
            $all_maps[$key]->map_kind = $map_version;
        }

        $license_info = Wp2leads_License::get_lecense_info();
        $license_email = $license_info['email'];
        $license_key = $license_info['key'];
        $site_url = Wp2leads_License::get_current_site();

        $parameters = array (
            'license_email' => $license_email,
            'license_key' => $license_key,
            'site_url' => $site_url,
            'event' => 'export',
            'maps'  =>  serialize($all_maps)
        );

        $request = wp_remote_post(
            base64_decode('aHR0cHM6Ly9tYXBzLndwMmxlYWRzLmNvbS9zZXJ2ZXIvbWFwcy5waHA='),
            array(
                'body'    => $parameters,
            )
        );

        $response = json_decode(wp_remote_retrieve_body( $request ), true);

        foreach ($response['body']['maps'] as $map_info) {
            $decoded_map_info = unserialize($map_info);

            foreach ($decoded_map_info as $map_id => $new_info) {
                $wpdb->query("
                    UPDATE " . $table . " SET info = '".$new_info."'
                    WHERE id = ".$map_id);
            }

        }
    }

    public static function update_maps_on_server($maps) {
        global $wpdb;

        $table = $wpdb->prefix . 'wp2l_maps';
        $maps_to_export_count = count($maps);
        $maps_ids = array();
        $maps_version = array();

        foreach ($maps as $map) {
            $maps_ids[] = $map['mapId'];
            $maps_version[$map['mapId']] = $map['mapVersion'];
        }

        $string_placeholders = array_fill(0, $maps_to_export_count, '%d');
        $placeholders_for_maps_id = implode(', ', $string_placeholders);

        $all_maps = $wpdb->get_results($wpdb->prepare(
            "SELECT * FROM " . $table . " WHERE id IN ($placeholders_for_maps_id)",
            $maps_ids
        ));

        foreach ($all_maps as $key => $map_obj) {
            $id = (int) $map_obj->id;
            $map_obj_info = unserialize($map_obj->info);
            $map_public_id = $map_obj_info['publicMapId'];
            $map_version = !empty($maps_version[$id]) ? $maps_version[$id] : '';

            $all_maps[$key]->public_map_id = $map_public_id;
            $all_maps[$key]->map_kind = $map_version;
        }

        $license_info = Wp2leads_License::get_lecense_info();
        $license_email = $license_info['email'];
        $license_key = $license_info['key'];
        $site_url = Wp2leads_License::get_current_site();

        $parameters = array (
            'license_email' => $license_email,
            'license_key' => $license_key,
            'site_url' => $site_url,
            'event' => 'update',
            'maps'  =>  serialize($all_maps)
        );

        $request = wp_remote_post(
            base64_decode('aHR0cHM6Ly9tYXBzLndwMmxlYWRzLmNvbS9zZXJ2ZXIvbWFwcy5waHA='),
            array(
                'body'    => $parameters,
            )
        );

        $response = json_decode(wp_remote_retrieve_body( $request ), true);

        foreach ($response['body']['maps'] as $map_info) {
            $decoded_map_info = unserialize($map_info);

            foreach ($decoded_map_info as $map_id => $new_info) {
                $wpdb->query("
                    UPDATE " . $table . " SET info = '".$new_info."'
                    WHERE id = ".$map_id);
            }
        }
    }

    public static function check_map_status_on_server($map_id) {
        global $wpdb;
        $table = $wpdb->prefix . 'wp2l_maps';

        $map = MapsModel::get($map_id);
        $map_info = $map->info;
        $decoded_map_info = unserialize($map_info);
        $public_map_id = $decoded_map_info['publicMapId'];
        $license_info = Wp2leads_License::get_lecense_info();
        $license_email = $license_info['email'];
        $license_key = $license_info['key'];
        $site_url = Wp2leads_License::get_current_site();

        $parameters = array (
            'license_email' => $license_email,
            'license_key' => $license_key,
            'site_url' => $site_url,
            'event' => 'check_status',
            'map'  =>  $public_map_id
        );

        $request = wp_remote_post(
            base64_decode('aHR0cHM6Ly9tYXBzLndwMmxlYWRzLmNvbS9zZXJ2ZXIvbWFwcy5waHA='),
            array(
                'body'    => $parameters,
            )
        );

        $response = json_decode(wp_remote_retrieve_body( $request ), true);

        if (404 === $response['code']) {
            $decoded_map_info['publicMapId'] = '';
            $decoded_map_info['publicMapHash'] = '';
            $decoded_map_info['publicMapContent'] = '';
            $decoded_map_info['publicMapOwner'] = '';
            $decoded_map_info['publicMapKind'] = '';
            $decoded_map_info['publicMapStatus'] = '';

            $wpdb->query("
                UPDATE " . $table . " SET info = '".serialize($decoded_map_info)."'
                WHERE id = ".$map_id);

            return false;

        } elseif (200 === $response['code']) {
            $status = $response['body']['status'];
            $decoded_map_info['publicMapStatus'] = $status;

            $wpdb->query("
                UPDATE " . $table . " SET info = '".serialize($decoded_map_info)."'
                WHERE id = ".$map_id);

            return $status;
        }
    }

    public static function get_available_maps_from_server() {
        $license_info = Wp2leads_License::get_lecense_info();
        $license_email = $license_info['email'];
        $license_key = $license_info['key'];
        $site_url = Wp2leads_License::get_current_site();

        $parameters = array (
            'license_email' => $license_email,
            'license_key' => $license_key,
            'site_url' => $site_url,
            'event' => 'get_available'
        );

        $request = wp_remote_post(
            base64_decode('aHR0cHM6Ly9tYXBzLndwMmxlYWRzLmNvbS9zZXJ2ZXIvbWFwcy5waHA='),
            array(
                'body'    => $parameters,
            )
        );

        $response = json_decode(wp_remote_retrieve_body( $request ), true);

        if (200 !== $response['code']) {
            return false;
        }

        $maps = $response['body']['maps'];

        if (count($maps) === 0) {
            return false;
        }

        return $maps;
    }

    public static function get_pending_maps_from_server() {
        $license_info = Wp2leads_License::get_lecense_info();
        $license_email = $license_info['email'];
        $license_key = $license_info['key'];
        $site_url = Wp2leads_License::get_current_site();

        $parameters = array (
            'license_email' => $license_email,
            'license_key' => $license_key,
            'site_url' => $site_url,
            'event' => 'get_pending'
        );

        $request = wp_remote_post(
            base64_decode('aHR0cHM6Ly9tYXBzLndwMmxlYWRzLmNvbS9zZXJ2ZXIvbWFwcy5waHA='),
            array(
                'body'    => $parameters,
            )
        );

        $response = json_decode(wp_remote_retrieve_body( $request ), true);

        if (200 !== $response['code']) {
            return false;
        }

        $maps = $response['body']['maps'];

        if (count($maps) === 0) {
            return false;
        }

        return $maps;
    }

    public static function upload_maps_to_server($maps) {
        $license_info = Wp2leads_License::get_lecense_info();

        $license_email = $license_info['email'];
        $license_key = $license_info['key'];
        $site_url = Wp2leads_License::get_current_site();

        $parameters = array(
            'license_email' => $license_email,
            'license_key' => $license_key,
            'site_url' => $site_url,
            'event' => 'export',
            'maps'  =>  serialize($maps)
        );

        $request = wp_remote_post(
            base64_decode('aHR0cHM6Ly9tYXBzLndwMmxlYWRzLmNvbS9zZXJ2ZXIvbWFwcy5waHA='),
            array(
                'body'    => $parameters,
            )
        );

        $response = json_decode(wp_remote_retrieve_body( $request ), true);
    }
}