<?php
/**
 * Created by PhpStorm.
 * User: oleksii.khodakivskyi
 * Date: 06.09.18
 * Time: 23:31
 */

class MapsModel {
    /**
     * Table name in the database
     *
     * @var string
     */
    private static $table_name = 'wp2l_maps';

    public static function get_table() {
        global $wpdb;

        return $wpdb->prefix . self::$table_name;
    }

    public static function get( $map_id ) {
        global $wpdb;

        if (!$map_id) {
            return null;
        }

        $table = self::get_table();

        return $wpdb->get_row("SELECT * FROM $table WHERE id=$map_id");
    }

    /**
     * Create New Map
     */
    public static function create( $data ) {
        global $wpdb;

        $info_array = array();
        $domain = Wp2leads_Admin::get_site_domain();
        $info_array['domain'] = $domain;
        $info_array['search'] = stripslashes($data['search']);
        $info_array['searchTable'] = stripslashes($data['searchTable']);
        $info_array['publicMapId'] = '';
        $info_array['publicMapHash'] = '';
        $info_array['publicMapContent'] = '';
        $info_array['publicMapOwner'] = '';
        $info_array['publicMapKind'] = '';
        $info_array['publicMapStatus'] = '';
        $info_array['publicMapVersion'] = '';

        if (!empty( $data['map'] ) && !empty( $data['api'] )) {

            $mapping = stripslashes($data['map']);
            $api = stripslashes($data['api']);

            $result = $wpdb->insert(
                self::get_table(),
                [
                    'time' => date('Y-m-d H:i:s'),
                    'name' => sanitize_text_field($data['name']),
                    'mapping' => serialize(json_decode($mapping, true)),
                    'api' => $api,
                    'info'  => serialize($info_array),
                ],
                ['%s', '%s', '%s', '%s', '%s']
            );

        } else if (!empty($data['map'])) {
            $mapping = stripslashes($data['map']);
            // create
            $result = $wpdb->insert(
                self::get_table(),
                [
                    'time' => date('Y-m-d H:i:s'),
                    'name' => sanitize_text_field($data['name']),
                    'mapping' => serialize(json_decode($mapping, true)),
                    'info'  => serialize($info_array),
                ],
                ['%s', '%s', '%s']
            );
        }

        if (!$result) {
            return false;
        }

        return array( 'mapping' => $mapping, 'map_id' => $wpdb->insert_id );
    }

    /**
     * Create New Map
     */
    public static function update( $data ) {
        global $wpdb;

        $map_id = $data['map_id'];

        if (!empty( $data['map'] ) && !empty( $data['api'] )) {
            $mapping = stripslashes($data['map']);
            $api = stripslashes($data['api']);

            $info_array['domain'] = $data['domain'];
            $info_array['search'] = stripslashes($data['search']);
            $info_array['searchTable'] = stripslashes($data['searchTable']);
            $info_array['publicMapId'] = stripslashes($data['publicMapId']);
            $info_array['publicMapHash'] = stripslashes($data['publicMapHash']);
            $info_array['publicMapContent'] = stripslashes($data['publicMapContent']);
            $info_array['publicMapKind'] = stripslashes($data['publicMapKind']);
            $info_array['publicMapOwner'] = stripslashes($data['publicMapOwner']);
            $info_array['publicMapStatus'] = stripslashes($data['publicMapStatus']);
            $info_array['publicMapVersion'] = stripslashes($data['publicMapVersion']);
            // update
            $result = $wpdb->update(
                self::get_table(),
                [
                    'time' => date('Y-m-d H:i:s'),
                    'name' => sanitize_text_field($data['name']),
                    'mapping' => serialize(json_decode($mapping, true)),
                    'api' => serialize(json_decode($api, true)),
                    'info'  => serialize($info_array)
                ],
                [
                    'id' => $map_id
                ],
                ['%s', '%s', '%s', '%s', '%s'],
                ['%d']
            );
        } else if ( !empty( $data['map'] ) ) {
            $mapping = stripslashes($data['map']);

            $info_array['domain'] = $data['domain'];
            $info_array['search'] = stripslashes($data['search']);
            $info_array['searchTable'] = stripslashes($data['searchTable']);
            $info_array['publicMapId'] = stripslashes($data['publicMapId']);
            $info_array['publicMapHash'] = stripslashes($data['publicMapHash']);
            $info_array['publicMapContent'] = stripslashes($data['publicMapContent']);
            $info_array['publicMapKind'] = stripslashes($data['publicMapKind']);
            $info_array['publicMapOwner'] = stripslashes($data['publicMapOwner']);
            $info_array['publicMapStatus'] = stripslashes($data['publicMapStatus']);
            $info_array['publicMapVersion'] = stripslashes($data['publicMapVersion']);
            // update
            $result = $wpdb->update(
                self::get_table(),
                [
                    'time' => date('Y-m-d H:i:s'),
                    'name' => sanitize_text_field($data['name']),
                    'mapping' => serialize(json_decode($mapping, true)),
                    'info'  => serialize($info_array),
                ],
                [
                    'id' => $map_id
                ],
                ['%s', '%s', '%s'],
                ['%d']
            );
        } else if ( !empty( $data['api'] ) ) {
            $mapping = stripslashes($data['api']); // Map to API

            $api_to_save = json_decode($mapping, true);

            $api_to_save = serialize($api_to_save);

            $result = $wpdb->update(
                self::get_table(),
                [
                    'time' => date('Y-m-d H:i:s'),
                    'api' => $api_to_save
                ],
                [
                    'id' => $map_id
                ],
                ['%s','%s'],
                ['%d']
            );
        }

        if (!$result) {
            return false;
        }

        return array( 'mapping' => $mapping, 'map_id' => $map_id );
    }

    public static function delete( $map_id ) {
        global $wpdb;

        return false !== $wpdb->delete(self::get_table(), ['id' => $map_id], ['%d']);
    }

    /**
     * @param $map
     * @param int $limit
     * @param int $offset
     *
     * @return array|bool|null|object
     */
    public static function get_map_query_results($map, $limit = 100, $offset = 0) {
        global $wpdb;

        $all_columns = $map['selects'];
        $excluded_columns = !empty($map['excludes']) ? $map['excludes'] : false;

        if ($excluded_columns) {
            $map_columns = array_diff($all_columns, $excluded_columns);
        } else {
            $map_columns = $all_columns;
        }

        $memory = (memory_get_usage()/1048576);

        $map['full_selects'] = self::get_selections_for_query($map);

        $mapSerialize = serialize($map);

        $query = self::generate_map_query($map, $limit, $offset);

        if (empty($query)) {
            return false;
        }

        $results = $wpdb->get_results($query);
        $v_names = array();

        foreach ($map_columns as $map_column_name) {
            if(strpos($map_column_name, 'v.') === 0) {
                $v_names[] = $map_column_name;
            }
        }

        $memory = (memory_get_usage()/1048576);

        // error_log('Memory Usage: ' . $memory);

        if(isset($map['virtual_relationships']) && count($map['virtual_relationships']) > 0) {
            foreach($results as $i => $result) {
                foreach ($map['virtual_relationships'] as $vr) {
                    $table_from_key = explode('.', $vr['column_key'])[0];
                    $table_from_value = explode('.', $vr['column_value'])[0];
                    $column_key_name = explode('.', $vr['column_key'])[1];
                    $column_key_value = explode('.', $vr['column_value'])[1];

                    $from = ' FROM ';

                    if ($table_from_key === $table_from_value) {
                        $from .= $wpdb->prefix . $table_from_key;
                    } else {
                        $from .= $wpdb->prefix . $table_from_key . ', ' . $wpdb->prefix . $table_from_value;
                    }

                    $field_value = isset($result->{$vr['table_from'] . '.' . $vr['column_from']}) ? $result->{$vr['table_from'] . '.' . $vr['column_from']} : NULL;
                    $compare = $wpdb->prefix . $vr['table_to'] . '.' . $vr['column_to'];

                    if (is_null($field_value)) {
//                        $field_value = isset($result->{$vr['table_to'] . '.' . $vr['column_to']}) ? $result->{$vr['table_to'] . '.' . $vr['column_to']} : NULL;
//                        $compare = $wpdb->prefix . $vr['table_from'] . '.' . $vr['column_from'];

                        $result_sub = false;
                    } else {
                        $where = 'WHERE ';
                        if (!is_null($field_value)) {
                            $where .= $compare . '=' . $field_value;

                            $sql = "SELECT {$wpdb->prefix}{$vr['column_key']}, {$wpdb->prefix}{$vr['column_value']} {$from} {$where};";
                        } else {
                            $where .= '1=1';

                            $sql = "SELECT DISTINCT {$wpdb->prefix}{$vr['column_key']} {$from} {$where};";
                        }

                        $result_sub = $wpdb->get_results($sql);
                    }

                    if (!empty($result_sub)) {
                        foreach ($result_sub as $item) {
                            $column_post_name = $item->{$column_key_name};
                            $column_post_name = str_replace($wpdb->prefix, '', $column_post_name);
                            $v_column_name = 'v.' . $table_from_key . '-' . $column_post_name;

                            if (isset($map['excludes']) && is_array($map['excludes'])) {
                                if(in_array($v_column_name, $map['excludes'])) {
                                    continue;
                                }
                            }

                            if (property_exists($item, $column_key_value)) {
                                $value = self::checkValue( $item->{$column_key_value} );
                            } else {
                                $value = '';
                            }

                            $v_names[] = $v_column_name;

                            if (isset($results[$i]->{$v_column_name}) && !empty($value)) {
                                if (strpos($results[$i]->{$v_column_name}, $value) === false) {
                                    $results[$i]->{$v_column_name} .= $value .' ';
                                }
                            } else {
                                $results[$i]->{$v_column_name} = $value .' ';
                            }
                        }

                        $v_names = array_unique($v_names);
                    }

                    unset($result_sub);
                }

                unset($result);
            }
        }

        if (!empty($_GET['tab']) && 'map_to_api' === $_GET['tab']) {
            error_log('Memory Usage map_to_api: ' . $memory);
        }

        // error_log('Memory Usage: ' . $memory);

        $count = count($results);

        for($i = 0; $i < $count; $i++) {
            if (isset($map['comparisons']) && count($map['comparisons'])) {
                foreach ($map['comparisons'] as $comparisons) {
                    if(!property_exists($results[$i], $comparisons['tableColumn'])) {
                        unset($results[$i]);
                        continue 2;
                    } else {
                        if (isset($comparisons['conditions'])) {
                            $value = trim(self::checkValue($results[$i]->{$comparisons['tableColumn']}));
                            $passed = false;

                            foreach ($comparisons['conditions'] as $condition) {
                                switch ($condition['operator']) {
                                    case 'like':
                                        if (trim($value) === $condition['string']) {
                                            $passed = true;
                                        }
                                        break;
                                    case 'not-like':
                                        if (trim($value) !== $condition['string']) {
                                            $passed = true;
                                        }
                                        break;
                                    case 'contains':
                                        if (strpos(strtolower($value), strtolower($condition['string'])) !== false) {
                                            $passed = true;
                                        }
                                        break;
                                    case 'not contains':
                                        if (strpos(strtolower($value), strtolower($condition['string'])) === false) {
                                            $passed = true;
                                        }
                                        break;
                                }
                            }

                            if (!$passed) {
                                unset($results[$i]);
                                continue 2;
                            }
                        }
                    }
                }
            }

            foreach($v_names as $v_name) {
                if(!property_exists($results[$i], $v_name)) {
                    $results[$i]->{$v_name} = '';
                }
            }

            $tmp = json_decode(json_encode($results[$i]), true);
            ksort($tmp);
            $results[$i] = (object)$tmp;

            unset($tmp);
        }

        if (!empty($_GET['tab']) && 'map_to_api' === $_GET['tab']) {
            // error_log('Memory Usage map_to_api: ' . $memory);
        }

        $results = array_values($results);

        return $results;
    }

    public static function get_map_query_rows_count($map) {
        global $wpdb;

        $map['count'] = true;
        $query = MapsModel::generate_map_query($map, NULL, NULL);

        if (empty($query)) {
            return 0;
        }

        $results = $wpdb->get_results($query);

        return $results;
    }

    public static function generate_map_query($map, $limit = 100, $offset = 0) {
        global $wpdb;
        $values = array();
        $concat_columns = isset($map['groupConcat']) ? is_array($map['groupConcat']) ? $map['groupConcat']: array($map['groupConcat']) : null;
        $concat = '';

        if (!empty($map['full_selects']['selects'])) {
            if (isset($map['excludes']) && !empty($map['excludes'])) {
                foreach ($map['excludes'] as $exclude) {
                    list($table, $column) = explode('.', $exclude);

                    if (isset($map['full_selects']['selects'][$table])) {
                        $key = array_search ($column, $map['full_selects']['selects'][$table]);

                        if ($key !== false) {
                            unset($map['full_selects']['selects'][$table][$key]);
                        }
                    }
                }
            }

            foreach ($map['full_selects']['selects'] as $table => $columns) {
                foreach ($columns as $column) {
                    $tableColumn = $table . '.' . $column;
                    if (!empty($concat_columns) && in_array($tableColumn, $concat_columns)) {
                        $tableColumnPrefixed = $wpdb->prefix . $table . '.' . $column;
                        $concat .= "GROUP_CONCAT(DISTINCT {$tableColumnPrefixed} ORDER BY {$tableColumnPrefixed} ASC SEPARATOR ', ') AS '{$tableColumn}(concatenated)', ";
                    } else {
                        // mysql specifier
                        array_push($values, $wpdb->prefix . $table);
                        array_push($values, $column);
                        // label the data
                        array_push($values, $table);
                        array_push($values, $column);
                    }
                }
            }

            $selectPlaceholders = implode(',', array_fill(0, count($values)/4, "`%s`.`%s` as '%s.%s'"));
        } elseif(isset($map['count']) && $map['count'] == true) {
            $selectPlaceholders = 'COUNT(*)';
        } else {
            // if, for some reason, all columns have been excluded...
            $selectPlaceholders = '*';
        }

        if (!isset($map['from'])) {
            return '';
        }

        $fromPlaceholder = '`%s`';
        array_push($values, $wpdb->prefix . $map['from']);

        $sql = "SELECT {$concat} {$selectPlaceholders} FROM {$fromPlaceholder} ";

        if (isset($map['joins']) && count($map['joins'])) {
            foreach ($map['joins'] as $join) {
                if($join['joinTable'] !== $unindexedTable = self::unindexed_table_name($join['joinTable'])) {
                    // table has nickname...
                    $sql .= "LEFT JOIN `%s` as `%s` ON `%s`.`%s` = `%s`.`%s` ";
                    // there are 6 placeholders with nickname...
                    array_push($values, $wpdb->prefix . $unindexedTable);
                    array_push($values, $wpdb->prefix . $join['joinTable']);
                    array_push($values, $wpdb->prefix . $join['joinTable']);
                    array_push($values, $join['joinColumn']);
                    array_push($values, $wpdb->prefix . $join['referenceTable']);
                    array_push($values, $join['referenceColumn']);
                } else {
                    // table has no nickname...
                    $sql .= "LEFT JOIN `%s` ON `%s`.`%s` = `%s`.`%s` ";
                    // for every join statement, there are 5 placeholders
                    array_push($values, $wpdb->prefix . $join['joinTable']);
                    array_push($values, $wpdb->prefix . $join['joinTable']);
                    array_push($values, $join['joinColumn']);
                    array_push($values, $wpdb->prefix . $join['referenceTable']);
                    array_push($values, $join['referenceColumn']);
                }
            }
        }

        $comparisons = "WHERE 1=1 ";

        if (isset($map['comparisons']) && count($map['comparisons'])) {

            foreach ($map['comparisons'] as $comparison) {
                if (strpos($comparison['tableColumn'], 'v.') === false) {
                    list($table, $column) = explode('.', $comparison['tableColumn']);

                    if (isset($comparison['conditions'])) {
                        $connect_string = 'AND';
                        foreach ($comparison['conditions'] as $condition) {
                            $operator = '';
                            $value = trim($condition['string']);
                            switch ($condition['operator']) {
                                case 'like':
                                    $operator = '=';
                                    break;
                                case 'not-like':
                                    $operator = '!=';
                                    break;
                                case 'contains':
                                    $operator = 'LIKE';
                                    $value = '%' . trim($condition['string']) . '%';
                                    break;
                                case 'not contains':
                                    $operator = 'NOT LIKE';
                                    $value = '%' . trim($condition['string']) . '%';
                                    break;
                            }

                            $table_column = "`{$wpdb->prefix}{$table}`.`{$column}`";
                            $comparisons .= " {$connect_string} %s %s '%s'";

                            array_push($values, $table_column);
                            array_push($values, $operator);
                            array_push($values, $value);

                            $connect_string = 'OR';
                        }
                    }
                }
            }
        }

        $sql .= $comparisons;

        if (!empty($map['keyBy'])) {
            $sql .= ' GROUP BY %s';
            array_push($values, $wpdb->prefix . $map['keyBy']);
        }

        $sql .= ' ORDER BY `%s`.`%s` DESC';
        array_push($values, $wpdb->prefix . $map['from']);
        array_push($values, self::fetch_primary_key_for_table($map['from']));

        if (!is_null($limit)) {
            $sql .= " LIMIT %d";
            array_push($values, $limit);
        }

        if (!is_null($offset)) {
            $sql .= " OFFSET %d";
            array_push($values, $offset);
        }

        return vsprintf($sql, $values);
    }

    public static function get_selections_for_query( $map ) {
        $selects = array(
            'selects' => array(),
            'v_columns_counter' => 0
        );

        if ($map['selects']) {
            $v_columns = 0;
            foreach ($map['selects'] as $select) {
                list($table, $column) = explode('.', $select);

                if($table == 'v') {
                    $v_columns++;
                    continue;
                }

                if (!isset($selects['selects'][$table])) {
                    $selects['selects'][$table] = array();
                }

                $selects['selects'][$table][] = $column;
            }

            if (isset($map['joins']) && count($map['joins'])) {
                foreach ($map['joins'] as $join) {
                    if (!array_key_exists($join['joinTable'], $selects['selects'])) {
                        $selects['selects'][$join['joinTable']] = array();
                    }

                    $joining_table_columns = self::fetch_columns_for_table($join['joinTable']);

                    foreach ($joining_table_columns as $column) {
                        if (!in_array($column, $selects['selects'][$join['joinTable']])) {
                            $selects['selects'][$join['joinTable']][] = $column;
                        }
                    }
                }
            }

            $selects['v_columns_counter'] = $v_columns;
        }

        return $selects;
    }

    public static function fetch_columns_for_table( $table ) {
        global $wpdb;

        $table = self::unindexed_table_name( $table );

        return array_map(function ($item) {
            return $item->Field;
        }, $wpdb->get_results('DESCRIBE ' . $wpdb->prefix . $table . ';'));
    }

    public static function fetch_all_columns_for_map($options) {
        global $wpdb;

        if (!$options['is_new_map']) {
            $map = unserialize(MapsModel::get($options['map_id'])->mapping);
        } else {
            $map = $options['new_map'];
        }

        $map['full_selects'] = MapsModel::get_selections_for_query($map);

        $mapSerialize = serialize($map);

        $query = MapsModel::generate_map_query($map, NULL, NULL);

        if (empty($query)) {
            return false;
        }

        $results = $wpdb->get_results($query);
        $v_names = array();

        foreach ($map['full_selects']['selects'] as $table => $columns) {
            foreach ($columns as $column) {
                $b_columns[] = $table . '.' . $column;
            }
        }

        $main_table_columns = MapsModel::fetch_columns_for_table($map['from']);

        foreach ($main_table_columns as $column) {
            if (!in_array($column, $b_columns)) {
                $b_columns[] = $map['from'] . '.' . $column;
            }
        }

        if(isset($map['virtual_relationships']) || !empty($map['virtual_relationships'])) {
            foreach($results as $i => $result) {
                foreach ($map['virtual_relationships'] as $vr) {
                    $table_from_key = explode('.', $vr['column_key'])[0];
                    $table_from_value = explode('.', $vr['column_value'])[0];
                    $column_key_name = explode('.', $vr['column_key'])[1];
                    $column_key_value = explode('.', $vr['column_value'])[1];

                    $from = ' FROM ';

                    if ($table_from_key === $table_from_value) {
                        $from .= $wpdb->prefix . $table_from_key;
                    } else {
                        $from .= $wpdb->prefix . $table_from_key . ', ' . $wpdb->prefix . $table_from_value;
                    }

                    $field_value = isset($result->{$vr['table_from'] . '.' . $vr['column_from']}) ? $result->{$vr['table_from'] . '.' . $vr['column_from']} : NULL;
                    $compare = $wpdb->prefix . $vr['table_to'] . '.' . $vr['column_to'];

                    if (is_null($field_value)) {
                        $field_value = isset($result->{$vr['table_to'] . '.' . $vr['column_to']}) ? $result->{$vr['table_to'] . '.' . $vr['column_to']} : NULL;
                        $compare = $wpdb->prefix . $vr['table_from'] . '.' . $vr['column_from'];
                    }

                    $where = 'WHERE ';
                    if (!is_null($field_value)) {
                        $where .= $compare . '=' . $field_value;
                    } else {
                        $where .= '1=1';
                    }

                    $sql = <<<EOL
                        SELECT 
                            {$wpdb->prefix}{$vr['column_key']},
                            {$wpdb->prefix}{$vr['column_value']}
                        {$from}
                        {$where};
EOL;

                    $result_sub = $wpdb->get_results($sql);

                    if (!empty($result_sub)) {
                        foreach ($result_sub as $item) {
                            $v_names[] = 'v.' . $table_from_key . '-' . $item->{$column_key_name};
                        }
                    }
                }
            }
        }

        $columns = array_values(array_unique(array_merge($b_columns, $v_names)));

        return $columns;
    }

    /**
     * @param $table
     *
     * @return mixed
     */
    public static function unindexed_table_name($table) {
        @list($table, $index) = explode('-', $table);
        return $table;
    }

    public static function fetch_primary_key_for_table($table)
    {
        global $wpdb;

        $table = self::unindexed_table_name($table);

        $results = $wpdb->get_results('DESCRIBE ' . $wpdb->prefix . $table . ';');

        $onlyPriKey = array_filter($results, function ($item) {
            return $item->Key == "PRI";
        });

        if (count($onlyPriKey)) {
            return $onlyPriKey[0]->Field;
        } else {
            return null;
        }
    }

    public static function checkValue($checking_value) {
        if ( !is_serialized( $checking_value ) ) {
            return $checking_value;
        }

        $value = @unserialize($checking_value);

        if (empty($value)) {
            $value = $checking_value;
        } else {
            if (is_array($value)) {
                $value = ApiHelper::arrayToString($value);

                $value = implode(',', $value);
            } else {
                $value = '';
            }
        }

        return $value;
    }

}