<?php
/**
 * Created by PhpStorm.
 * User: oleksii.khodakivskyi
 * Date: 06.09.18
 * Time: 23:24
 */

class StatisticsManager {

    /**
     * Filtering and saving statistics data using StatisticsModel
     *
     * @param $data
     */
    public static function saveStatistics($data) {
        if (!empty($data)) {
            $data['time'] = date('Y-m-d H:i:s', $data['time']);
            $data['statistics'] = json_encode($data['statistics']);
            StatisticsModel::insert($data);
        }
    }

    public static function getTotallyTransferedData($map_id) {
        $result = StatisticsModel::get( array( 'map_id' => $map_id ), array( 'statistics' ) );

        $times = StatisticsModel::get( array( 'map_id' => $map_id, 'transfer_type' => 'manually' ), array( 'time' ) );
        $crontimes = StatisticsModel::get( array( 'map_id' => $map_id, 'transfer_type' => 'cron' ), array( 'time' ) );
        $last_time = '';
        $last_crontime = '';
        foreach ($times as $time) {
            $last_time = $time['time'];

            break;
        }

        foreach ($crontimes as $time) {
            $last_crontime = $time['time'];

            break;
        }

        $transfered = array(
            'unique' => 0,
            'all'  => 0,
            'time'  => $last_time,
            'crontime'  => $last_crontime,
        );

        foreach ( $result as $statistic ) {
            $decoded_statistic = json_decode( $statistic['statistics'], true );

            $transfered['unique'] += (int) $decoded_statistic['New subscribers'];
            $transfered['all'] += (int) $decoded_statistic['Total transferred'];
        }

        return $transfered;
    }

    public static function getLastTotallyTransferedData($map_id) {
        $result = StatisticsModel::get( array( 'map_id' => $map_id ), array( 'statistics' ) );

        $transfered = array(
            'unique' => 0,
            'all'  => 0,
        );

        foreach ( $result as $statistic ) {
            $decoded_statistic = json_decode( $statistic['statistics'], true );

            $transfered['unique'] += (int) $decoded_statistic['New subscribers'];
            $transfered['all'] += (int) $decoded_statistic['Total transferred'];
        }

        return $transfered;
    }

    /**
     * Returnign the list of all available maps
     *
     * @return array
     */
    public static function getMaps() {
        global $wpdb;
        $maps = array();
        $table_name = $wpdb->prefix . 'wp2l_maps';
        $result = $wpdb->get_results("SELECT id, name FROM {$table_name} ORDER BY id ASC", ARRAY_A);

        foreach ($result as $row) {
            $maps[$row['id']] = array(
                'name' => $row['name']
            );
        }

        return $maps;
    }

    /**
     * Converting UNIX time to the Local time using time zone configured in the WP Settings
     *
     * @param $time
     * @return string
     */
    public static function convertTimeToLocal($time) {
        $format = 'Y-m-d H:i:s';
        $time_zone = get_option('timezone_string');
        $dt = new DateTime();
        $dt->setTimestamp(strtotime($time));
        $dt->setTimezone(new DateTimeZone($time_zone));
        return $dt->format($format);
    }
}