<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @since             1.0.0
 * @package           Wp2leads
 *
 * @wordpress-plugin
 * Plugin Name:       WP2Leads
 * Description:       Transfer user data from nearly all WordPress Plugins to Klick-Tipp
 * Version:           1.0.1.7
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       wp2leads
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'WP2LEADS_VERSION', '1.0.1.7' );
define( 'WP2LEADS_DEBUG', false);

if ( ! defined( 'WP2LEADS_PLUGIN_FILE' ) ) {
    define( 'WP2LEADS_PLUGIN_FILE', __FILE__ );
}

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-wp2leads-activator.php
 */
function activate_wp2leads() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-wp2leads-activator.php';
	Wp2leads_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-wp2leads-deactivator.php
 */
function deactivate_wp2leads() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-wp2leads-deactivator.php';
	Wp2leads_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_wp2leads' );
register_deactivation_hook( __FILE__, 'deactivate_wp2leads' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-wp2leads.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_wp2leads() {

	$plugin = new Wp2leads();
	$plugin->run();

}
run_wp2leads();
